package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.entity.Contract;
import com.bocloud.cos.service.AttachmentService;
import com.bocloud.cos.service.ContractService;
import com.bocloud.cos.utils.FileUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.jsonwebtoken.lang.Assert;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

/**
 * 合同管理
 */
@Slf4j
@RestController
@RequestMapping("/v1/contract")
@Tag(name = "合同管理")
public class ContractController {

    @Value("${spring.upload.path:/home/cmp}")
    private String path;

    @Autowired
    private ContractService contractService;

    @Autowired
    private AttachmentService attachmentService;

    @Operation(summary = "获取合同列表")
    @GetMapping
    public GeneralResult<GridBean<Contract>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return contractService.list(pager, context);
    }

    @Operation(summary = "获取合同详情")
    @GetMapping("/{id}")
    public GeneralResult<Contract> detail(@PathVariable(Common.ID) Long id) {
        return contractService.detail(id);
    }

    @Operation(summary = "新增合同")
    @PostMapping
    public GeneralResult<Void> create(@RequestBody Contract contract, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return contractService.create(contract, context);
    }

    @Operation(summary = "删除合同")
    @DeleteMapping("/{id}")
    public GeneralResult<Void> delete(@PathVariable(Common.ID) Long id) {
        return contractService.delete(id);
    }

    @Operation(summary = "修改合同状态")
    @PostMapping("/update_status")
    public GeneralResult<Void> updateStatus(@RequestParam(value = "id") Long id,
                                            @RequestParam(value = "status") String status,
                                            @RequestParam(value = "attachment", required = false) MultipartFile file,
                                            @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            Attachment attachment = attachmentService.upload(file, context);
            Long attachmentId = null;
            if (null != attachment && attachment.getId() != null) {
                attachmentId = attachment.getId();
            }
            return contractService.updateStatus(id, status, attachmentId, context);
        } catch (Exception e) {
            log.error("修改合同状态失败：{}", e);
            return new GeneralResult<>(true, "状态修改失败");
        }
    }

    @Operation(summary = "导出合同")
    @GetMapping("/download")
    public GeneralResult<Void> download(@RequestParam(value = "attachmentId") Long attachmentId,
                                        HttpServletRequest request, HttpServletResponse response) {
        try {
            // 获取合同
            Attachment attachment = attachmentService.get(attachmentId);
            Assert.isTrue(null != attachment && null != attachment.getPath(), "该合同找不到对应附件，无法导出");

            String charsetCode = String.valueOf(StandardCharsets.UTF_8);
            File file = new File(path + "/" + attachment.getPath());
            //中文乱码解决
            String type = request.getHeader("User-Agent").toLowerCase();
            String fileName = attachment.getName();
            // 字符编码格式
            if (type.indexOf("firefox") > 0 || type.indexOf("chrome") > 0) {
                //谷歌或火狐
                fileName = new String(fileName.getBytes(charsetCode), "iso8859-1");
            } else {
                //IE
                fileName = URLEncoder.encode(fileName, charsetCode);
            }
            // 设置响应的头部信息
            response.setHeader("content-disposition", "attachment;filename=" + fileName);
            // 设置响应内容的类型
            response.setContentType(FileUtil.fileContentType(fileName) + "; charset=" + charsetCode);
            // 设置响应内容的长度
            response.setContentLength((int) file.length());
            // 输出
            FileUtil.outStream(Files.newInputStream(file.toPath()), response.getOutputStream());
        } catch (Exception e) {
            log.error("文件下载异常{}", e);
        }
        return new GeneralResult<>(true, "");
    }

}