package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.ServiceAddress;
import com.bocloud.cos.intf.ServiceAddressService;
import com.bocloud.cos.params.ServiceAddressParams;
import com.bocloud.cos.entity.internel.CloudVendor;
import com.bocloud.cos.entity.internel.Department;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/services/address")
@Tag(name = "服务商管理")
@Slf4j
public class ServiceAddressController {

    private final ServiceAddressService serviceAddressService;

    public ServiceAddressController(ServiceAddressService serviceAddressService) {
        this.serviceAddressService = serviceAddressService;
    }

    /**
     * 获取服务列表
     *
     * @param pager 分页信息
     * @return 分页结果
     */
    @Operation(summary = "获取服务商列表")
    @GetMapping
    public GeneralResult<GridBean<ServiceAddress>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceAddressService.list(pager, context);
    }

    /**
     * 平台配置
     *
     * @param id      id
     * @param params  配置信息
     * @param context 操作人
     * @return 操作结果
     */
    @Operation(summary = "平台配置")
    @PutMapping("/{id}/config")
    public Result configVendor(@PathVariable(Common.ID) Long id,
                               @RequestBody ServiceAddressParams params,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceAddressService.configVendor(id, params.getVendorId(), context.getTarget());
    }

    /**
     * 添加服务商
     *
     * @param params  对象信息
     * @param context 操作人
     * @return 操作结果
     */
    @Operation(summary = "添加服务商")
    @PostMapping
    public Result create(@RequestBody ServiceAddressParams params,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceAddressService.create(params, context);
    }

    /**
     * 编辑服务商
     *
     * @param id      id
     * @param params  编辑信息
     * @param context 操作人信息
     * @return 操作结果
     */
    @Operation(summary = "修改服务商信息")
    @PutMapping("/{id}")
    public Result modify(@RequestBody ServiceAddressParams params,
                         @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setId(id);
        return serviceAddressService.modify(params, context.getTarget());
    }

    /**
     * 移除服务
     *
     * @param id      id
     * @param context 操作人信息
     * @return 操作结果
     */
    @Operation(summary = "移除服务商")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceAddressService.remove(id, context.getTarget());
    }

    /**
     * 查询详情
     *
     * @param id id
     * @return 详情
     */
    @Operation(summary = "查询服务商详情")
    @GetMapping("/{id}")
    public GeneralResult<ServiceAddress> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceAddressService.detail(id);
    }

    /**
     * 查询服务商平台
     *
     * @return 列表
     */
    @Operation(summary = "查询服务商平台")
    @GetMapping("/{id}/vendor")
    public GeneralResult<List<CloudVendor>> vendorList(@PathVariable(value = Common.ID) Long id) {
        return serviceAddressService.listVendors(id);
    }

    /**
     * 查询组织架构平台
     *
     * @return 列表
     */
    @Operation(summary = "查询组织架构平台")
    @GetMapping("/depart")
    public GeneralResult<List<Department>> departList() {
        return serviceAddressService.listDepart();
    }


    @Deprecated
    @Operation(summary = "导出服务商列表")
    @RequestMapping(value = "/download", method = {RequestMethod.GET})
    public void export(Pager pager, HttpServletRequest request, HttpServletResponse response, @Value(Common.REQ_CONTEXT) RequestContext context) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = serviceAddressService.export(paramList, context);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "服务商数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            log.error("export service bill error,", e);
        }
    }
}
