package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.cos.intf.ServiceBillService;
import com.bocloud.cos.model.LineChartResult;
import com.bocloud.cos.params.BillActionParams;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * 账单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@RestController
@RequestMapping("/v1/bills")
@Tag(name = "账单管理")
@Slf4j
public class ServiceBillController {

    private final ServiceBillService serviceBillService;

    public ServiceBillController(ServiceBillService serviceBillService) {
        this.serviceBillService = serviceBillService;
    }

    @Operation(summary = "获取账单列表")
    @GetMapping
    public GeneralResult<GridBean<ServiceBill>> list(Pager pager, @RequestParam(value = "chargeMode", required = false) String chargeMode,
                                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.list(pager, chargeMode, context);
    }

    @Operation(summary = "获取账单列表总金额")
    @GetMapping("/sum/money")
    public GeneralResult sumMoneyOfList(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.sumMoneyOfList(pager, context);
    }

    @Operation(summary = "字典查询账单列表")
    @GetMapping("/condition")
    public GeneralResult listByCondition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.listByCondition(condition, context);
    }

    @Operation(summary = "添加账单")
    @PostMapping
    public Result create(
            @RequestBody ServiceBill serviceBill, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.create(serviceBill, context);
    }

    @Operation(summary = "修改账单")
    @PutMapping("/{id}")
    public Result modify(
            @PathVariable(value = Common.ID) Long id, @RequestBody ServiceBill serviceBill,
            @Value(Common.REQ_CONTEXT) RequestContext context) {
        serviceBill.setId(id);
        return serviceBillService.modify(serviceBill, context);
    }

    @Operation(summary = "资源放入回收站生成账单")
    @PutMapping
    public Result recycleBill(
            @RequestBody ServiceBill serviceBill) {
        if (null == serviceBill) {
            return new GeneralResult<>(false, "参数格式错误");
        }
        return serviceBillService.recycleBill(serviceBill.getSn());
    }

    @Operation(summary = "删除账单")
    @DeleteMapping("/{id}")
    public Result remove(
            @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.remove(id, context);
    }

    @Deprecated
    @Operation(summary = "导出账单列表")
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager,
                       @Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request,
                       HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = serviceBillService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "账单统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            log.error("export service bill error,", e);
        }
    }

    @Operation(summary = "查看账单详情")
    @GetMapping("/{id}")
    public GeneralResult<ServiceBill> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceBillService.detail(id);
    }

    @Operation(summary = "获取账单明细列表")
    @GetMapping("/items")
    public GeneralResult<GridBean<BillItem>> items(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.items(pager, context);
    }

    @Operation(summary = "添加账单明细")
    @PostMapping("/items")
    public Result createItem(@RequestBody BillItem billItem, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.createItem(billItem, context);
    }

    @Operation(summary = "修改账单明细")
    @PutMapping("/items/{id}")
    public Result modifyItem(@PathVariable(value = Common.ID) Long id, @RequestBody BillItem item,
                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        item.setId(id);
        return serviceBillService.modifyItem(item, context);
    }

    /**
     * 删除账单明细
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "删除账单明细")
    @DeleteMapping("/items/{id}")
    public Result removeItem(@PathVariable(value = Common.ID) Long id,
                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.removeItem(id, context);
    }

    /**
     * 查看账单明细详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "查看账单明细详情")
    @GetMapping("/items/{id}")
    public GeneralResult<BillItem> detailItem(@PathVariable(value = Common.ID) Long id) {
        return serviceBillService.detailItem(id);
    }

    /**
     * 支付账单/取消账单支付
     *
     * @param action
     * @param context
     * @return
     */
    @Operation(summary = "支付账单/取消账单支付")
    @PatchMapping
    public Result action(
            @RequestBody BillActionParams action, @Value(Common.REQ_CONTEXT) RequestContext context) {

        switch (action.getAction()) {
            case pay:
                return serviceBillService.pay(action.getSn(), action.getCouponId(), context);
            case cancel:
                return serviceBillService.cancel(action.getSn(), context);
            default:
                return new Result(false, "暂不支持此操作");
        }
    }

    @Operation(summary = "查询消费账单概览")
    @GetMapping("/stats")
    public GeneralResult<LineChartResult> stats(@RequestParam(value = Common.CODE) String code,
                                                @RequestParam(value = "range") String range, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.stats(code, range, context);
    }

    @Operation(summary = "查询消费账单概览")
    @GetMapping("/report")
    public GeneralResult report(@RequestParam(value = Common.CODE) String code,
                                @RequestParam(value = "vendorType") String vendorType, @RequestParam(value = "range") String range,
                                @RequestParam(value = "tenantId") Long tenantId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceBillService.report(code, vendorType, range, tenantId, context);
    }

    @Operation(summary = "查询消费账单报表")
    @GetMapping("/dashboard")
    public GeneralResult dashboard(@RequestParam(value = Common.CODE, required = false) String code,
                                   @RequestParam(value = "start", required = false) String start,
                                   @RequestParam(value = "end", required = false) String end,
                                   @RequestParam(value = "city", required = false) String city,
                                   @RequestParam(value = "managerId", required = false) Long managerId,
                                   @RequestParam(value = "tenantId", required = false) Long tenantId) {
        return serviceBillService.dashboard(code, start, end, city, managerId, tenantId);
    }
}
