package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.intf.ServiceSkuService;
import com.bocloud.cos.params.SkuActionParams;
import com.bocloud.cos.params.SkuParams;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author admin
 */
@RestController
@RequestMapping("/v1/services/skus")
@Tag(name = "服务产品管理")
public class ServiceSkuController {

    private final ServiceSkuService serviceSkuService;

    public ServiceSkuController(ServiceSkuService serviceSkuService) {
        this.serviceSkuService = serviceSkuService;
    }

    /**
     * 获取服务产品列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(summary = "获取服务产品列表")
    public GeneralResult<GridBean<ServiceSku>> list(Pager pager) {
        return serviceSkuService.list(pager);
    }

    /**
     * 查看服务产品详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "查看服务产品详情")
    public GeneralResult<ServiceSku> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceSkuService.detail(id);
    }

    /**
     * 添加服务产品
     *
     * @param sku
     * @param context
     * @return
     */
    @PostMapping
    @Operation(summary = "添加服务产品")
    public Result create(@RequestBody SkuParams sku,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.create(sku, context);
    }

    /**
     * 修改服务产品
     *
     * @param id
     * @param sku
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改服务产品")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody SkuParams sku, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.modify(id, sku, context);
    }

    /**
     * 删除服务产品
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除服务产品")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.remove(id, context.getTarget());
    }

    /**
     * 批量上下架服务产品
     *
     * @param params
     * @param context
     * @return
     */
    @PatchMapping("/batch")
    @Operation(summary = "批量上下架服务产品")
    public Result batchUpdate(@RequestBody SkuActionParams params,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (params.getAction()) {
            case enable:
            case disable:
                return serviceSkuService.updateStatus(params.getIds(), params.getAction().name(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }

    /**
     * 根据规格查询服务产品
     *
     * @param spec
     * @param categoryId
     * @return
     */
    @GetMapping("/getbyspec")
    @Operation(summary = "根据规格查询服务产品")
    public GeneralResult<ServiceSku> getBySpec(@RequestParam(value = "spec") String spec,
                                               @RequestParam(value = "categoryId") Long categoryId) {
        return serviceSkuService.getBySpec(spec, categoryId);
    }

    /**
     * 获取服务产品列表
     *
     * @param pager
     * @return
     */
    @GetMapping("/condition")
    @Operation(summary = "获取服务产品列表")
    public GeneralResult<List<ServiceSku>> listByCondition(Pager pager) {
        return serviceSkuService.list(pager.getParams());
    }

}
