package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceApplicationService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.ResourceEventPublisher;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.entity.internel.Tenant;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author weiwei
 */
@RequiredArgsConstructor
@Slf4j
@Component
public class FlowEventListener {

    private final AmqpTemplate amqpTemplate;
    private final ResourceApplicationService resourceApplyService;
    private final ServiceOrderRepository orderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final OrderItemRepository orderItemRepository;
    private final SmsTenantService tenantService;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final ResourceEventPublisher resourceEventPublisher;
    private final CouponRepository couponRepository;

    @RabbitListener(queues = "bocloud.flow.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (!StringUtils.isNotEmpty(content)) {
                return;
            }
            log.info("收到审核结果消息：{}", content);
            JSONObject object = JSONObject.parseObject(content, JSONObject.class);
            if (null == object || object.isEmpty()) {
                log.warn("content is null or empty");
                return;
            }
            operate(object);
        } catch (Exception e) {
            log.error("flow callback error: {}", e.getMessage(), e);
        }
    }

    /**
     * 处理任务类审批消息
     *
     * @param object 参数
     */
    private void operate(JSONObject object) {
        String category = object.getString("ApplyCategory");
        String status = object.getString("ApplyStatus");
        if (BocloudStatus.Service.APPROVED.name().equals(status)) {// 审批通过
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam applicationParam = JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder applyOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext applyContext = JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    applicationParam.setContext(applyContext);
                    applicationParam.setOrder(null);
                    applyOrderHandle(applyOrder, applicationParam);
                    break;
                case AlterationOperate:
                    //变更审批后的参数
                    AlterationParam alterationParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterationOrder =
                            JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext context =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    AlterationParam oldParams =
                            JSONObject.parseObject(alterationOrder.getApplyParam(), AlterationParam.class);
                    alterationParam.setContext(context);
                    alterationParam.setExpiredTime(oldParams.getExpiredTime());
                    alterationOrderHandle(alterationOrder, alterationParam);
                    break;
                case ExtensionOperate:
                    ExtensionParam extensionParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    extensionParam.setContext(extensionContext);
                    extensionOrderHandle(extensionOrder, extensionParam);
                    break;
                case UnsubscribeOperate:
                    CancelationParam cancelationParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), CancelationParam.class);
                    ServiceOrder cancelationOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext cancelationContext =
                            JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    cancelationParam.setContext(cancelationContext);
                    unsubscribeOrderHandle(cancelationOrder, cancelationParam);
                    break;
                case ResourceComputerClone:
                    CloneParams config = JSONObject.parseObject(object.getString("ApplyParams"), CloneParams.class);
                    RequestContext cloneContext =
                            JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    try {
                        resourceApplyService.clone(config, cloneContext);
                    } catch (RuntimeException e) {
                        NotifyMessage message = new NotifyMessage(cloneContext, "服务实例【" + config.getInstanceName() + "】克隆失败通知",
                                e.getMessage(), true, false);
                        amqpTemplate.convertAndSend("event.notice.queue", message);
                    }
                    break;
                default:
                    break;
            }
        } else if (BocloudStatus.Service.UNRATIFIED.name().equals(status)) {// 审批不通过
            Boolean isCancel = object.getBoolean("ApplyIsCancel");
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam param =
                            JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder order = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    RequestContext context =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> items = orderItemRepository.listByOrderId(order.getId());
                    if (order.getServiceCode().startsWith("kvs.") || order.getServiceCode().startsWith("rds.") || order
                            .getServiceCode().startsWith("cms.")) {
                        //软件安装特殊处理 回调处理配额那边软件安装直回调一个消息处理
                        items.removeIf(orderItem -> !(orderItem.getServiceCode().startsWith("kvs.") || orderItem.getServiceCode()
                                .startsWith("rds.") || orderItem.getServiceCode().startsWith("cms.")));
                    }
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    String applyMsg = "订购失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过");
                    // 取消审批的订单
                    items.forEach(item -> {
                        item.setStatus(OrderItemStatus.OpenedFailure);
                        // item.setOrderId(null);
                        item.setErrorMsg(applyMsg);
                        orderItemRepository.update(item);
                    });

                    // 站内信
                    String subject = "服务实例【" + param.getConfigs().getString("name") + "】订购失败通知";
                    NotifyMessage orderMessage = new NotifyMessage(context, subject, applyMsg, true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", orderMessage);

                    // 弹窗通知
                    resourceEventPublisher.send(
                            new OperateResult(false, applyMsg, "resource." + param.getConfigs().getString("category").toLowerCase(), null,
                                    OperateResult.OperateCategory.ResourceEvent, context));

                    break;
                case AlterationOperate:
                    AlterationParam alteration =
                            JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    alterOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(alterOrder);
                    RequestContext alterationContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    AlterationParam oldParams =
                            JSONObject.parseObject(alterOrder.getApplyParam(), AlterationParam.class);
                    alteration.setExpiredTime(oldParams.getExpiredTime());
                    String alterMsg = "变更失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过");
                    List<OrderItem> alterItems = orderItemRepository.listByOrderId(alterOrder.getId());
                    for (OrderItem alterItem : alterItems) {
                        alterItem.setStatus(OrderItemStatus.ChangeFailure);
                        alterItem.setErrorMsg(alterMsg);
                        orderItemRepository.update(alterItem);
                    }

                    // 站内信
                    String alterSubject = "服务实例【" + alteration.getConfigs().getString("name") + "】变更失败通知";
                    NotifyMessage alterMessage = new NotifyMessage(alterationContext, alterSubject, "资源申请失败：配额不足", true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", alterMessage);

                    // 弹窗通知
                    resourceEventPublisher.send(
                            new OperateResult(false, alterMsg, "resource." + alteration.getConfigs().getString("category").toLowerCase(), null,
                                    OperateResult.OperateCategory.ResourceEvent, alterationContext));
                    break;
                case ExtensionOperate:
                    ExtensionParam extension =
                            JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> extensionItems = orderItemRepository.listByOrderId(extensionOrder.getId());
                    //批量更新订单项
                    extensionItems.forEach(orderItem -> {
                        orderItem.setStatus(OrderItemStatus.ExtensionFailure);
                        orderItem.setErrorMsg("延期失败：" + ((isCancel != null && isCancel) ? "用户已取消" : "审批未通过"));
                        orderItemRepository.update(orderItem);
                    });
                    extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(extensionOrder);
                    NotifyMessage message =
                            new NotifyMessage(extensionContext, "服务实例【" + extension.getInstanceName() + "】延期失败通知", (isCancel != null && isCancel) ? "用户已取消" : "审批未通过",
                                    true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                    break;
                case UnsubscribeOperate:
                    CancelationParam cancelationParam =
                            JSONObject.parseObject(object.getString("ApplyParams"), CancelationParam.class);
                    ServiceOrder cancelationOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext cancelationContext =
                            JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> cancelationItems = orderItemRepository.listByOrderId(cancelationOrder.getId());
                    //批量更新订单项
                    cancelationItems.forEach(orderItem -> {
                        orderItem.setStatus(OrderItemStatus.UnsubscribeFailure);
                        String errorMsg = "退订失败：审批未通过";
                        if (isCancel != null && isCancel) {
                            errorMsg = "退订失败：用户已取消";
                        }
                        orderItem.setErrorMsg(errorMsg);
                        orderItemRepository.update(orderItem);
                    });
                    cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(cancelationOrder);
                    NotifyMessage cancelationMessage =
                            new NotifyMessage(cancelationContext, "服务实例【" + cancelationParam.getInstanceName() + "】退订失败通知",
                                    (isCancel != null && isCancel) ? "用户已取消" : "审批未通过",
                                    true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", cancelationMessage);
                default:
                    break;
            }
        }
    }

    private void extensionOrderHandle(ServiceOrder extensionOrder, ExtensionParam extensionParam) {
        try {
            ExtensionParam param = JSONObject.parseObject(extensionOrder.getApplyParam(), ExtensionParam.class);
            //延期月份
            Integer extension = extensionParam.getExtension();
            DurationModel.Mode mode = DurationModel.Mode.MONTH;
            param.setExtension(extensionParam.getExtension());
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(extensionOrder.getId());
            for (OrderItem orderItem : orderItems) {
                DurationModel duration = JSONObject.parseObject(orderItem.getDuration(), DurationModel.class);
                DurationModel durationModel = new DurationModel();
                mode = duration.getMode();
                durationModel.setMode(duration.getMode());
                durationModel.setAmount(extension);
                orderItem.setDuration(JSONObject.toJSONString(durationModel));
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(orderItem.getEffectiveTime());
                if (duration.getMode().equals(DurationModel.Mode.MONTH)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                orderItem.setExpiredTime(calendar.getTime());
                orderItem.setStatus(OrderItemStatus.Extensioning);
                orderItemRepository.update(orderItem);
            }
            if (mode.equals(DurationModel.Mode.YEAR)) {
                param.setExtension(extensionParam.getExtension() * 12);
            }
            extensionOrder.setApplyParam(JSONObject.toJSONString(param));
            extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            orderRepository.update(extensionOrder);
        } catch (Exception e) {
            log.error("extension order handle error", e);
        }
    }

    private void alterationOrderHandle(ServiceOrder alterationOrder, AlterationParam params) {
        try {
            ResourceOperate alterationOperate = ResourceOperateFactory.getInstance(params.getService());
            //找之前待审批订单项 更新订单项
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(alterationOrder.getId());
            // 此处为流程回调处理类，所以肯定会有待审批订单项，如果拿不到，循环处理
            Integer temp = 3;
            while (auditItems.size() == 0 && temp > 0){
                Thread.sleep(1000L);
                auditItems = orderItemRepository.listAuditItems(alterationOrder.getId());
                temp --;
            }
            if (auditItems.size() == 0){
                // 未拿到的话可能会出现已经审批完成，但是订单明细中有数据处于审批中状态的订单项
                log.warn("未拿到有待审批订单项");
            }
            auditItems.forEach(item -> {
                item.setStatus(OrderItemStatus.Canceld);
                item.setOrderId(null);
                orderItemRepository.update(item);
            });

            OrderItem oldItem;
            Map<String, Integer> oldQuota;
            ElementModel elementModelObj = params.getUpgrade().get(0);
            String serviceCode = elementModelObj.getServiceCode();
            // 1. 根据资源sn找到资源《订购》的订单
            ServiceOrder applyOrder = orderRepository.queryOrderBySnAndType(params.getInstance(), ChargeCommon.Type.apply.name());
            if (null == applyOrder) {
                throw new IllegalArgumentException("旧订单未找到");
            }
            // 2。根据serviceCode找到变更资源的sn
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(applyOrder.getId(), serviceCode);
            if (null == orderItems || orderItems.isEmpty()) {
                throw new IllegalArgumentException("旧订单项未找到");
            }

            OrderItem applyItem = orderItems.get(0);
            List<ElementModel> applyElementModel = JSON.parseArray(applyItem.getElements(), ElementModel.class);
            Integer count = 1;
            if (null != applyElementModel && !applyElementModel.isEmpty()) {
                ElementModel elementModel = applyElementModel.get(0);
                count = elementModel.getCount();
            }
            // 3. 根据变更资源的sn找到最新有效的变更资源的订单项
            // 3.1 最近订单项
            oldItem = orderItemRepository.queryLastSuccessBySn(applyItem.getSn());
            if (null == oldItem) {
                throw new IllegalArgumentException("最近订单项未找到");
            }
            // 3.2 最近订单
            ServiceOrder oldOrder = orderRepository.query(oldItem.getOrderId());
            if (null == oldOrder) {
                throw new IllegalArgumentException("最近订单未找到");
            }
            // 4。生成旧配额
            oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));

            //生成新的订单项
            List<OrderItem> newAlterationItems = alterationOperate.buildAlterationItem(params.getService(), params.getInstance(),
                    params.getOperation(), params.getUpgrade(), params.getExpiredTime());
            Tenant tenant = tenantService.detail(alterationOrder.getTenantId());
            BigDecimal discount = tenant == null ? BigDecimal.valueOf(1L) : tenant.getDiscount();
//            Assert.notNull(tenant, "租户信息不存在");
            // 订单项设置折扣
            for (OrderItem newAlterationItem : newAlterationItems) {
                List<ElementModel> elementModelList =
                        JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                Integer finalCount = count;
                elementModelList.forEach(elementModel -> {
                    elementModel.setRatio(discount);
                    elementModel.setCount(finalCount);
                });
                newAlterationItem.setElements(JSONObject.toJSONString(elementModelList));
                newAlterationItem.setFinalPrice(newAlterationItem.getBasicPrice().multiply(discount));
                newAlterationItem.setOrderId(alterationOrder.getId());
            }

            //配额处理
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(newAlterationItems);
            List<QuotaBean> quotaBeans = new ArrayList<>();
            // 变更操作 配额差值
            // 变更后订单项的配额

            log.info("变更后订单项的配额: {}", JSON.toJSONString(alterationQuota));
            log.info("变更前订单项得配额: {}", JSON.toJSONString(oldQuota));
            for (Map.Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String quotaCode = entry.getKey();
                // 旧配额
                Integer number = Optional.ofNullable(oldQuota.get(quotaCode)).orElse(0);
                // 新配额和旧配额得差值
                quotaBeans.add(new QuotaBean(quotaCode, alterationQuota.get(quotaCode) - number));
                alterationQuota.put(quotaCode, alterationQuota.get(quotaCode) - number);
            }
            // 校验配额并更新配额
            log.info("校验配额并且更新配额的量：" + JSON.toJSONString(alterationQuota));
            if (tenant != null) {
                GeneralResult<List<ResourceQuota>> result = resourceQuotaService.handleQuota(alterationQuota, tenant.getId(), false);
                //操作产生的配额
                params.setQuotas(quotaBeans);
                if (result.isFailed()) {
                    // 配额不足订单取消
                    alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(alterationOrder);
                    RequestContext alterationContext = params.getContext();
                    AlterationParam oldParams =
                            JSONObject.parseObject(alterationOrder.getApplyParam(), AlterationParam.class);
                    params.setExpiredTime(oldParams.getExpiredTime());

                    String msg = "变更失败：配额不足";
                    List<OrderItem> alterItems = orderItemRepository.listByOrderId(alterationOrder.getId());
                    for (OrderItem alterItem : alterItems) {
                        alterItem.setStatus(OrderItemStatus.ChangeFailure);
                        alterItem.setErrorMsg(msg);
                        orderItemRepository.update(alterItem);
                    }
                    // 通知用户
                    String subject = "服务实例【" + params.getInstanceName() + "】变更失败通知";
                    NotifyMessage message = new NotifyMessage(alterationContext, subject, msg, true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                }
                Assert.isTrue(result.isSuccess(), result.getMessage());
            }
            switch (params.getOperation()) {
                case Upgrade:
                    if (alterationOperate instanceof PaasResourceOperate) {
                        for (OrderItem alterationItem : newAlterationItems) {
                            List<ElementModel> elementModels =
                                    JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                            if ("vmware.storage.disk".equals(alterationItem.getServiceCode())) {
                                ElementModel diskElement = elementModels.get(0);
                                List<SpecModel> diskSpecModel = diskElement.getSpecs();
                                Integer disk =
                                        diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                                .reduce(Integer::sum).orElse(0);
                                params.setDataDisk(disk);
                            } else {
                                ElementModel upgradeElement = elementModels.get(0);
                                List<SpecModel> specs = upgradeElement.getSpecs();
                                Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                                BigDecimal memory = specs.stream()
                                        .map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                                params.setUpgradeSkuId(upgradeElement.getSkuId());
                                params.setCpu(cpu);
                                params.setMemory(memory);
                            }
                            alterationItem.setElements(JSONObject.toJSONString(elementModels));
                            alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(discount));
                            alterationItem.setOrderId(alterationOrder.getId());
                        }
                    } else {
                        // 获取升级规格 设置升级后的cpu内存大小到变更参数里面
                        OrderItem newAlterationItem = newAlterationItems.get(0);
                        List<ElementModel> elementModels =
                                JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                        ElementModel element = elementModels.get(0);
                        List<SpecModel> specs = element.getSpecs();
                        Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        BigDecimal memory =
                                specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                        params.setUpgradeSkuId(element.getSkuId());
                        params.setCpu(cpu);
                        params.setMemory(memory);
                        Integer systemDisk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        if (!systemDisk.equals(0)) {
                            params.setSystemDisk(systemDisk);
                        }
                    }
                    break;
                case EditDisk:
                    OrderItem editItem = newAlterationItems.get(0);
                    List<ElementModel> editDiskElements =
                            JSONArray.parseArray(editItem.getElements(), ElementModel.class);
                    editDiskElements.forEach(elementModel -> elementModel.setRatio(discount));
                    editItem.setElements(JSONObject.toJSONString(editDiskElements));
                    editItem.setFinalPrice(editItem.getBasicPrice().multiply(discount));
                    editItem.setOrderId(alterationOrder.getId());
                    break;
                case AddDisk:
                    int diskTotal = 0;
                    for (OrderItem alterationItem : newAlterationItems) {
                        List<ElementModel> diskElements =
                                JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                        ElementModel diskElement = diskElements.get(0);
                        List<SpecModel> diskSpecModel = diskElement.getSpecs();
                        Integer disk = diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                        diskTotal += disk;
                        // 订单项设置折扣
                        diskElements.forEach(elementModel -> elementModel.setRatio(discount));
                        alterationItem.setElements(JSONObject.toJSONString(diskElements));
                        alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(discount));
                        alterationItem.setOrderId(alterationOrder.getId());
                    }
                    params.setDataDisk(diskTotal);
                default:
                    break;
            }
            params.setCategory(ServiceResourceMapping.getResourceCategory(params.getService()).name());
            params.setOldStatus(oldItem.getStatus());
            // 保存新订单
            newAlterationItems.forEach(orderItemRepository::save);
            params.setNewInstance(newAlterationItems.get(0).getSn());
            params.setOrderSn(alterationOrder.getSn());
            alterationOrder.setApplyParam(JSONObject.toJSONString(params));
            // 预付费生成未支付账单
            if (ChargeCommon.ChargeMode.PreCharge == alterationOrder.getChargeMode()) {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                orderRepository.update(alterationOrder);
            } else {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(alterationOrder);
                if (alterationOperate instanceof PaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.modification.queue", params);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.modification.queue", params);
                }
            }
        } catch (Exception e) {
            log.error("alteration order handle error", e);
        }
    }

    /**
     * 申请订单审批处理(生成新订单项，配额更新)
     *
     * @param order            订单
     * @param applicationParam 订单申请参数
     */
    private void applyOrderHandle(ServiceOrder order, ApplicationParam applicationParam) {
        try {
            ResourceOperate applyOperate = ResourceOperateFactory.getInstance(order.getServiceCode());
            CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
            //找待审批订单（有流程的话就有待审批订单，需要更新订单项）
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(order.getId());
            if (!auditItems.isEmpty()) {
                //生成新订单项
                List<OrderItem> items = applyOperate.buildApplyItem(order.getServiceCode(), applicationParam.getEmption(), applicationParam.getElements());
                // 扣除配额
                Map<String, Integer> newQuota = OrderItemUtils.getItemQuota(items);
                GeneralResult<List<ResourceQuota>> result = resourceQuotaService.handleQuota(newQuota, order.getTenantId(), false);
                // 配额不足，取消订单，通知用户
                if (result.isFailed()) {
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    RequestContext context = applicationParam.getContext();
                    String msg = "变更失败：配额不足";
                    // 取消审批的订单
                    auditItems.forEach(item -> {
                        item.setStatus(OrderItemStatus.OpenedFailure);
                        // item.setOrderId(null);
                        item.setErrorMsg(msg);
                        orderItemRepository.update(item);
                    });

                    String subject = "服务实例【" + service.getName() + "】订购失败通知";
                    NotifyMessage message = new NotifyMessage(context, subject, "资源申请失败：配额不足", true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                }
                // 配额校验失败，直接返回
                Assert.isTrue(result.isSuccess(), result.getMessage());
                Assert.isTrue(!items.isEmpty(), "生成订单内容错误");
                Tenant tenant = tenantService.detail(order.getTenantId());
                Assert.notNull(tenant, "租户信息不存在");
                if (!ListTool.isEmpty(items)) {
                    BigDecimal discount = BigDecimal.ONE;
                    Coupon coupon = couponRepository.getCouponByOrderSn(order.getSn());
                    if (null != coupon && null != coupon.getDiscount()) {
                        discount = coupon.getDiscount().multiply(BigDecimal.valueOf(0.1));
                    }
                    BigDecimal tenantDiscount = tenant.getDiscount().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ONE : tenant.getDiscount();
                    tenant.setDiscount(tenantDiscount.multiply(discount));
                    for (OrderItem item : items) {
                        item.setOrderId(order.getId());
                        List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
                        if (elementModels.isEmpty()) {
                            ElementModel elementModel = new ElementModel();
                            elementModel.setRatio(tenant.getDiscount());
                            elementModels = Lists.newArrayList(elementModel);
                        } else {
                            elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        }
                        item.setElements(JSONObject.toJSONString(elementModels));
                        item.setFinalPrice(item.getBasicPrice().multiply(tenant.getDiscount()));
                        orderItemRepository.save(item);
                    }
                    auditItems.forEach(item -> {
                        item.setStatus(OrderItemStatus.Canceld);
                        item.setOrderId(null);
                        orderItemRepository.update(item);
                    });

                }
                order.setItems(items);
            }
            JSONObject configs = applicationParam.getConfigs();
            if (null == configs) {
                configs = new JSONObject();
            }
            Long skuId = null;
            ServiceSku sku = null;
            for (ElementModel element : applicationParam.getElements()) {
                if (service.getCode().equals(element.getServiceCode())) {
                    skuId = element.getSkuId();
                    sku = serviceSkuRepository.query(skuId);
                    break;
                }
            }
            ServiceCategory category = null;
            configs.put("skuId", skuId);
            if (null != sku) {
                configs.put("skuName", sku.getName());
                category = serviceCategoryRepository.query(sku.getCategoryId());
            }
            if (null != category) {
                configs.put("categoryId", category.getId());
                configs.put("categoryCode", category.getCode());
                configs.put("categoryProps", category.getProps());
            }
            configs.put("category", ServiceResourceMapping.getResourceCategory(order.getServiceCode()).name());
            if (StringUtils.isNotEmpty(service.getTemplateName())) {
                configs.put("templateName", service.getTemplateName());
            }
            applicationParam.setService(service.getCode());
            applicationParam.setConfigs(configs);
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            //年计费下发到底层服务特殊处理
            EmptionModel emption = applicationParam.getEmption();
            DurationModel duration = emption.getDuration();
            DurationModel.Mode mode = duration.getMode();
            //下发到底层服务的时候按年计费时间做特殊处理
            if (mode == DurationModel.Mode.YEAR) {
                EmptionModel paramEmption = applicationParam.getEmption();
                paramEmption.getDuration().setAmount(paramEmption.getDuration().getAmount() * 12);
                paramEmption.getDuration().setMode(DurationModel.Mode.MONTH);
                applicationParam.setEmption(paramEmption);
            }
            order.setApplyParam(JSONObject.toJSONString(applicationParam));
            //后付费
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(order);
                applicationParam.setOrder(order);
                if ("vmware.nsx.vpc".equalsIgnoreCase(service.getCode())) {
                    log.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                    amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
                } else {
                    List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                    log.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                    if (applyOperate instanceof PaasResourceOperate) {
                        amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                    } else if (applyOperate instanceof IaasResourceOperate) {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    } else {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    }
                }
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                orderRepository.update(order);
            }
        } catch (Exception e) {
            log.error("apply order handle error", e);
        }
    }

    /**
     * 退订处理
     *
     * @param cancelationOrder 退订订单
     * @param cancelationParam 退订参数
     */
    private void unsubscribeOrderHandle(ServiceOrder cancelationOrder, CancelationParam cancelationParam) {
        try {
            ResourceOperate cancelationOperate = ResourceOperateFactory.getInstance(cancelationOrder.getServiceCode());
            cancelationOrder.setApplyParam(JSONObject.toJSONString(cancelationParam));
            cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
            orderRepository.update(cancelationOrder);
            List<OrderItem> extensionItems = orderItemRepository.listByOrderId(cancelationOrder.getId());
            for (OrderItem orderItem : extensionItems) {
                orderItem.setStatus(OrderItemStatus.Unsubscribing);
                orderItemRepository.update(orderItem);
            }
            if (cancelationOperate instanceof PaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.deletion.queue", cancelationParam);
            } else {
                if ("vmware.nsx.vpc".equals(cancelationOrder.getServiceCode())) {
                    amqpTemplate.convertAndSend("network.resource.deletion.queue", cancelationParam);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.deletion.queue", cancelationParam);
                }
            }
        } catch (Exception e) {
            log.error("cancelation order handle error", e);
            String subject = "服务实例【" + cancelationParam.getInstanceName() + "】退订失败通知";
            NotifyMessage message = new NotifyMessage(cancelationParam.getContext(), subject, e.getMessage(), true, false);
            amqpTemplate.convertAndSend("event.notice.queue", message);
        }
    }
}
