package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.cos.intf.RecycledService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.model.OrderMessage.Operation;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.ResourceEventPublisher;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.entity.internel.User;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.OperateResult.OperateCategory;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

import static com.bocloud.cos.enums.OrderItemStatus.ChangeFailure;
import static com.bocloud.cos.enums.OrderItemStatus.ChangeSuccess;

/**
 * 此处用来处理资源申请成功之后订单状态，预付费订单的账单逻辑
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月13日
 */
@RequiredArgsConstructor
@Slf4j
@Component
public class ResourceResultListener {

    private final CloudServiceRepository serviceRepository;
    private final ServiceOrderRepository serviceOrderRepository;
    private final OrderItemRepository orderItemRepository;
    private final ServiceBillRepository billRepository;
    private final BillItemRepository billItemRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final ChargeItemRepository chargeItemRepository;
    private final ResourceEventPublisher resourceEventPublisher;
    private final AmqpTemplate amqpTemplate;
    private final TenantAccountService accountService;
    private final RecycledService recycledService;
    private final SmsUserService smsUserService;
    private final CouponRepository couponRepository;

    /**
     * 站内信通知队列
     */
    private static final String NOTICE_QUEUE = "event.notice.queue";
    /**
     * 资源操作结果队列
     */
    private static final String RES_RESULT_QUEUE = "resource.result.queue";

    @RabbitListener(admin = "rabbitAdmin", queues = RES_RESULT_QUEUE, autoStartup = "true")
    public void onMessage(OrderMessage message) {
        try {
            switch (message.getOperation()) {
                case Application:
                    applicationHandel(message);
                    break;
                case Extension:
                    //延期要成功都成功
                    extensionHandel(message);
                    break;
                case Cancelation:
                    cancelHandel(message);
                    break;
                case Upgrade:
                case ScaleOut:
                case ScaleIn:
                    alterationHandel(message);
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("Get error message：", e);
        }
    }

    /**
     * 申请回调处理
     *
     * @param message 订单回调消息
     */
    private void applicationHandel(OrderMessage message) throws Exception {
        if (null == message || !StringUtils.isNotEmpty(message.getSn())) {
            log.info("Order message sn is empty!");
            return;
        }
        log.info("接收到资源申请回调消息：{}", JSONObject.toJSON(message));
        // 处理子订单状态
        OrderItem item = orderItemRepository.queryBySn(message.getSn());
        if (null == item) {
            log.info("Order item does not exists!");
            return;
        }
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        if (null == order) {
            log.info("Service order does not exists!");
            return;
        }
        // paas资源特殊处理
        if (item.getServiceCode().startsWith("kvs.") || item.getServiceCode().startsWith("rds.") ||
                item.getServiceCode().startsWith("cms.") || item.getServiceCode().equals("qingcloud.standard.server")) {
            passApplyHandler(message, item, order);
            return;
        }
        item.setErrorMsg(message.getMessage());
        item.setStatus(message.getStatus());
        //更新订单项
        item.setEffectiveTime(message.getEffectiveTime());
        item.setExpiredTime(message.getExpiredTime());
        if (!ListTool.isEmpty(message.getResources())) {
            item.setResources(JSONObject.toJSONString(message.getResources()));
        }
        orderItemRepository.update(item);
        List<OrderItem> orderItems = orderItemRepository.listByOrderId(order.getId());

        if (OrderItemStatus.OpenedSuccess.equals(message.getStatus())) {
            if (ChargeCommon.ChargeMode.PostCharge.equals(order.getChargeMode())) {
                //非公有云 后付费添加到跑批计费内容中
                CloudService cloudService = serviceRepository.queryByCode(item.getServiceCode());
                if (!cloudService.getIsPublic()) {
                    ChargeItem chargeItem = new ChargeItem();
                    chargeItem.setServiceCode(item.getServiceCode());
                    chargeItem.setPrice(item.getFinalPrice());
                    chargeItem.setGmtCreate(new Date());
                    chargeItem.setItemSn(item.getSn());
                    chargeItem.setItemId(item.getId());
                    chargeItem.setDeleted(false);
                    chargeItem.setType(ChargeItem.Type.loop);
                    chargeItemRepository.baseSave(chargeItem);
                }
            }
            //修改订单状态
            if (orderItems.stream()
                    .anyMatch(orderItem -> orderItem.getStatus().equals(OrderItemStatus.OpenedSuccess))) {
                order.setResourceStatus(ChargeCommon.ResourceStatus.ready);
                serviceOrderRepository.update(order);
            }
            resourceEventPublisher.send(
                    new OperateResult(true, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
            // 发送站内信
        } else {
            // 取消使用的优惠券
            couponRepository.updateStatus(Coupon.Status.unused.name(), order.getSn(), null);

            //如果开通失败 回滚配额
            List<QuotaBean> quotas = OrderItemUtils.getItemQuotaBeans(Lists.newArrayList(item));
            log.info("open failed call back quota,the quota is {}", JSONObject.toJSONString(quotas));
            resourceQuotaService.callBackQuota(quotas, order.getTenantId());
            // 回滚授信
            this.processTenantAccount(order);
            if (orderItems.stream()
                    .anyMatch(orderItem -> orderItem.getStatus().equals(OrderItemStatus.OpenedSuccess))) {
                order.setResourceStatus(ChargeCommon.ResourceStatus.failed);
            } else {
                order.setResourceStatus(ChargeCommon.ResourceStatus.exception);
            }
            serviceOrderRepository.update(order);
            resourceEventPublisher.send(
                    new OperateResult(false, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
            // 发送站内信
        }
        this.sendMessage(message, Operation.Application, item.getServiceCode(), message.getSn());
    }

    private void passApplyHandler(OrderMessage message, OrderItem item, ServiceOrder order) throws Exception {
        List<OrderItem> orderItems = orderItemRepository.listByOrderId(order.getId());
        if (OrderItemStatus.OpenedSuccess.equals(message.getStatus())) {
            for (OrderItem orderItem : orderItems) {
                orderItem.setErrorMsg(message.getMessage());
                orderItem.setStatus(message.getStatus());
                //更新订单项
                orderItem.setEffectiveTime(message.getEffectiveTime());
                orderItem.setExpiredTime(message.getExpiredTime());
                if (!ListTool.isEmpty(message.getResources())) {
                    orderItem.setResources(JSONObject.toJSONString(message.getResources()));
                }
                orderItemRepository.update(orderItem);
                if (ChargeCommon.ChargeMode.PostCharge.equals(order.getChargeMode())) {
                    ChargeItem chargeItem = new ChargeItem();
                    chargeItem.setServiceCode(orderItem.getServiceCode());
                    chargeItem.setPrice(orderItem.getFinalPrice());
                    chargeItem.setGmtCreate(new Date());
                    chargeItem.setItemSn(orderItem.getSn());
                    chargeItem.setItemId(orderItem.getId());
                    chargeItem.setDeleted(false);
                    chargeItem.setType(ChargeItem.Type.loop);
                    chargeItemRepository.baseSave(chargeItem);
                }
            }
            order.setResourceStatus(ChargeCommon.ResourceStatus.ready);
            resourceEventPublisher.send(
                    new OperateResult(true, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        } else {
            for (OrderItem orderItem : orderItems) {
                orderItem.setErrorMsg(message.getMessage());
                orderItem.setStatus(message.getStatus());
                //更新订单项
                orderItem.setEffectiveTime(message.getEffectiveTime());
                orderItem.setExpiredTime(message.getExpiredTime());
                if (!ListTool.isEmpty(message.getResources())) {
                    orderItem.setResources(JSONObject.toJSONString(message.getResources()));
                }
                orderItemRepository.update(orderItem);
            }
            //如果开通失败 回滚配额
            List<QuotaBean> quotas = OrderItemUtils.getItemQuotaBeans(orderItems);
            log.info("open failed call back quota,the quota is {}", JSONObject.toJSONString(quotas));
            resourceQuotaService.callBackQuota(quotas, order.getTenantId());
            // 回滚授信
            this.processTenantAccount(order);
            order.setResourceStatus(ChargeCommon.ResourceStatus.failed);
            resourceEventPublisher.send(
                    new OperateResult(false, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        }
        serviceOrderRepository.update(order);
        // 发送站内信
        this.sendMessage(message, Operation.Application, item.getServiceCode(), message.getSn());
    }

    /**
     * 变更回调处理
     *
     * @param message
     */
    private void alterationHandel(OrderMessage message) throws Exception {
        if (null == message || StringUtils.isEmpty(message.getNewSn())) {
            log.info("Order message is empty!");
            return;
        }
        log.info("接收到资源变更回调消息：{}", JSONObject.toJSON(message));
        OrderItem newItem = orderItemRepository.queryBySn(message.getNewSn());
        if (null == newItem) {
            log.info("Order item does not exists!");
            return;
        }
        ServiceOrder newOrder = serviceOrderRepository.query(newItem.getOrderId());
        if (null == newOrder) {
            log.info("Service order does not exists!");
            return;
        }
        Date effectiveTime = message.getEffectiveTime();
        if (null == effectiveTime) {
            effectiveTime = new Date();
        }
        newItem.setEffectiveTime(effectiveTime);
        List<ResourceModel> oldResources = new ArrayList<>();
        if (!StringUtils.isEmpty(newItem.getResources())) {
            oldResources = JSONArray.parseArray(newItem.getResources(), ResourceModel.class);
        }
        Map<String, ResourceModel> resourceMap = new HashMap<>(4);
        oldResources.forEach(o -> resourceMap.put(o.getUuid(), o));
        List<ResourceModel> upgrades = message.getResources();
        if (!ListTool.isEmpty(upgrades)) {
            for (ResourceModel model : upgrades) {
                resourceMap.put(model.getUuid(), model);
            }
        }

        newItem.setResources(JSONObject.toJSONString(resourceMap.values()));
        newItem.setStatus(message.getStatus());
        newItem.setErrorMsg(message.getMessage());
        orderItemRepository.update(newItem);

        if (ChangeFailure.getCode().equalsIgnoreCase(message.getStatus().getCode())) {
            newOrder.setResourceStatus(ChargeCommon.ResourceStatus.failed);
            serviceOrderRepository.update(newOrder);
            //如果变更失败 回滚配额
            List<QuotaBean> quotas = message.getQuotas();
            log.info("alteration failed call back quota,the quota is {}", JSONObject.toJSONString(quotas));
            resourceQuotaService.callBackQuota(quotas, newOrder.getTenantId());
            //回滚授信
            this.processTenantAccount(newOrder);
            resourceEventPublisher.send(
                    new OperateResult(false, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        } else if (ChangeSuccess.getCode().equalsIgnoreCase(message.getStatus().getCode())) {
            //变更成功计费项 计算最后一次费用 然后采用最新的item计费
            if (ChargeCommon.ChargeMode.PostCharge == newOrder.getChargeMode()) {
                //变更规格或者扩盘操作(对旧的计费做结账 再更新计费)
                ChargeItem chargeItem = chargeItemRepository.query(newItem.getSn());
                if (chargeItem != null) {
                    this.buildAlterationBill(message.getNewSn());
                    chargeItem.setItemSn(newItem.getSn());
                    chargeItem.setPrice(newItem.getFinalPrice());
                    chargeItem.setItemId(newItem.getId());
                    chargeItem.setGmtCreate(new Date());
                    chargeItemRepository.baseUpdate(chargeItem);
                } else {
                    //加盘操作（不用针对旧的记录结账直接加入计费项）
                    ChargeItem addDiskCharge = new ChargeItem();
                    addDiskCharge.setServiceCode(newItem.getServiceCode());
                    addDiskCharge.setPrice(newItem.getFinalPrice());
                    addDiskCharge.setGmtCreate(new Date());
                    addDiskCharge.setItemSn(newItem.getSn());
                    addDiskCharge.setItemId(newItem.getId());
                    addDiskCharge.setDeleted(false);
                    addDiskCharge.setType(ChargeItem.Type.loop);
                    chargeItemRepository.baseSave(addDiskCharge);
                }
            }
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(newOrder.getId());
            if (orderItems.stream().anyMatch(orderItem -> orderItem.getStatus().equals(ChangeSuccess))) {
                newOrder.setResourceStatus(ChargeCommon.ResourceStatus.ready);
                serviceOrderRepository.update(newOrder);
            }
            resourceEventPublisher.send(
                    new OperateResult(true, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        }
        // 发送站内信
        this.sendMessage(message, message.getOperation(), newItem.getServiceCode(), message.getNewSn());
    }

    /**
     * 延期回调处理
     *
     * @param message
     */
    private void extensionHandel(OrderMessage message) throws Exception {
        if (null == message || StringUtils.isEmpty(message.getSn())) {
            log.info("Order message is empty!");
            return;
        }
        log.info("接收到资源延期回调消息：{}", JSONObject.toJSON(message));
        // 处理子订单状态
        OrderItem item = orderItemRepository.queryBySn(message.getSn());
        if (null == item) {
            log.info("Order item does not exists!");
            return;
        }
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        if (null == order) {
            log.info("Service order does not exists!");
            return;
        }
        //延期要么都成功 要么都失败
        List<OrderItem> orderItems = orderItemRepository.listByOrderId(item.getOrderId());
        //延期成功
        if (message.getStatus().equals(OrderItemStatus.ExtensionSuccess)) {
            for (OrderItem orderItem : orderItems) {
                orderItem.setErrorMsg(message.getMessage());
                orderItem.setStatus(message.getStatus());
                orderItemRepository.update(orderItem);
            }
            resourceEventPublisher.send(
                    new OperateResult(true, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        } else {
            //延期失败
            for (OrderItem orderItem : orderItems) {
                orderItem.setErrorMsg(message.getMessage());
                orderItem.setStatus(message.getStatus());
                orderItemRepository.update(orderItem);
            }
            // 回滚授信
            this.processTenantAccount(order);
            resourceEventPublisher.send(
                    new OperateResult(false, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
            // 发送站内信
        }
        this.sendMessage(message, message.getOperation(), item.getServiceCode(), message.getSn());
    }

    /**
     * 退订回调处理
     *
     * @param message 订单结果消息
     */
    private void cancelHandel(OrderMessage message) {
        log.info("接收到资源退订回调消息：{}", JSONObject.toJSONString(message));
        Assert.isTrue(null != message && StringUtils.isNotBlank(message.getSn()), "Order message is empty!");
        OrderItem item = orderItemRepository.queryBySn(message.getSn());
        Assert.notNull(item, "Order item does not exists!");
        CloudService service = serviceRepository.queryByCode(item.getServiceCode());
        Assert.notNull(service, "订单明细关联的服务未找到，order item id:" + item.getId());
        if (OrderItemStatus.UnsubscribeSuccess.equals(message.getStatus())) {
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            Assert.notNull(order, "Service order does not exists!");
            // 移出回收站
            recycledService.remove(message.getSn(), message.getContext().getTarget());
            // 成功
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                // 删除产生后付费账单 最后一次结账
                this.buildCancelationBill(message.getSn());
                // 从计费项中移除
                chargeItemRepository.removeItem(message.getSn());
//            } else { //预付费（包年包月）退订时不创建账单，不退费用 edit by zhangyf 2022-03-23
//                if (service.getBillable()) {
//                    buildCancelationBillOfPre(message.getSn());
//                }
            }
            //如果退订成功 回滚配额
            List<QuotaBean> quotas;
            if (item.getServiceCode().startsWith("kvs.") || item.getServiceCode().startsWith("rds.") ||
                    item.getServiceCode().startsWith("cms.") || item.getServiceCode().equals("qingcloud.standard.server")) {
                quotas = message.getQuotas();
                List<OrderItem> diskItems = orderItemRepository.queryDiskSn(message.getSn());
                if (CollectionUtils.isNotEmpty(diskItems)) {
                    diskItems.stream().forEach(diskItem -> {
                        // 修改订单项状态
                        diskItem.setStatus(message.getStatus());
                        orderItemRepository.update(diskItem);

                        if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                            // 删除产生后付费账单 硬盘 最后一次结账
                            this.buildCancelationBill(diskItem.getSn());
                            // 硬盘从计费项中移除
                            chargeItemRepository.removeItem(diskItem.getSn());
                        }
                        //预付费（包年包月）退订时不创建账单，不退费用 edit by zhangyf 2022-03-23
//                        if (service.getBillable()) {
//                            buildCancelationBillOfPre(diskItem.getSn());
//                        }
                    });
                }
            } else {
                quotas = OrderItemUtils.getItemQuotaBeans(Lists.newArrayList(item));
            }
            item.setStatus(OrderItemStatus.UnsubscribeSuccess);
            orderItemRepository.update(item);
            List<OrderItem> items = orderItemRepository.listByOrderId(order.getId());
            boolean allUnsubscribeSuccess = items.stream().filter(i -> i.getStatus() != OrderItemStatus.UnsubscribeSuccess).count() == 0L;
            if (allUnsubscribeSuccess) {
                order.setType(ChargeCommon.Type.unsubscribe);
                serviceOrderRepository.update(order);
            }
            log.info("cancelation success call back quota,the quota is {}", JSONObject.toJSONString(quotas));
            resourceQuotaService.callBackQuota(quotas, order.getTenantId());
            resourceEventPublisher.send(
                    new OperateResult(true, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        } else {
            item.setStatus(OrderItemStatus.UnsubscribeFailure);
            orderItemRepository.update(item);
            resourceEventPublisher.send(
                    new OperateResult(false, message.getMessage(), "resource." + message.getCategory().toLowerCase(), null,
                            OperateCategory.ResourceEvent, message.getContext()));
        }
        // 发送站内信
        this.sendMessage(message, message.getOperation(), item.getServiceCode(), message.getSn());
    }

    /**
     * 退钱
     *
     * @param order
     */
    private void processTenantAccount(ServiceOrder order) {
        if (ChargeCommon.ChargeMode.PreCharge == order.getChargeMode()) {
            RequestContext requestContext =
                    new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(),
                            null);
            Long tenantId = order.getTenantId();
            if (null != order.getUserId() && 0L != order.getUserId()) {
                GeneralResult<User> userResult = smsUserService.detail(order.getUserId());
                if (userResult.isSuccess() && null != userResult.getData()) {
                    User user = userResult.getData();
                    tenantId = user.getTenantId();
                    requestContext.setTarget(user.getId());
                    requestContext.setTenant(user.getTenantId());
                    requestContext.setCatalog(Catalog.User);
                }
            }
            if (null != tenantId && 0L != tenantId) {
                List<ServiceBill> bills = billRepository.listByOrderId(order.getId());
                for (ServiceBill bill : bills) {
                    //退过钱的不再退钱
                    if (bill.getStatus().equals(ChargeCommon.BillStatus.canceld)) {
                        continue;
                    }
                    // 扣款
                    Result result = accountService
                            .accountAction(tenantId, bill.getId(), TenantAccount.Action.Callback, bill.getMoney(),
                                    requestContext);
                    Assert.isTrue(result.isSuccess(), result.getMessage());
                    bill.setStatus(ChargeCommon.BillStatus.canceld);
                    billRepository.update(bill);
                }
            }
        }
    }

    /**
     * 生成变更账单(后付费)
     *
     * @param sn
     */
    private void buildAlterationBill(String sn) {
        ChargeItem chargeItem = chargeItemRepository.query(sn);
        if (chargeItem == null) {
            //加盘操作 不需要作结账
            return;
        }
        OrderItem item = orderItemRepository.query(chargeItem.getItemId());
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        //查找当月账单
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
        if (null == bill) {
            bill =
                    new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                            order.getTenantId(), month, new Date());
            bill.setUserId(order.getUserId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            bill.setMoney(BigDecimal.ZERO);
            billRepository.save(bill);
        }
        BigDecimal oldMoney = bill.getMoney();
        // 对旧价格进行最后一次计费
        BigDecimal oldFinalPrice = chargeItem.getPrice();
        // 获取计费开始时间
        Date billStart = getBillStartDate();
        // 跑批开始时间(如果是0点之后跑批的 则从开始时间算到当前时间结束 今天之前跑批的 跑批从0点算到当前时间)
        Date chargeBegin = chargeItem.getGmtCreate();
        if (null != chargeBegin && chargeBegin.after(billStart)) {
            billStart = chargeBegin;
        }
        // 获取计费结束时间
        Date billEnd = new Date();
        BigDecimal billMoney = oldFinalPrice.divide(new BigDecimal(60), 20, RoundingMode.HALF_UP)
                .multiply(new BigDecimal((billEnd.getTime() - billStart.getTime()) / 60000));
        BillItem billItem = new BillItem();
        billItem.setBillId(bill.getId());
        billItem.setItemId(chargeItem.getItemId());
        //查找之前跑批的订单
        OrderItem chargeOrderItem = orderItemRepository.query(chargeItem.getItemId());
        billItem.setSkuId(chargeOrderItem.getSkuId());
        billItem.setChargeMode(order.getChargeMode());
        billItem.setPrice(billMoney);
        billItem.setBillStart(billStart);
        billItem.setBillEnd(billEnd);
        billItem.setMoney(billMoney);
        billItem.setGmtCreate(new Date());
        billItem.setCatalog(BillItem.Catalog.alteration);
        OrderItemUtils.elementNames(chargeOrderItem);
        billItem.setResource(JSONObject.toJSONString(chargeOrderItem.getElementList()));
        billItem.setUserId(order.getUserId());
        billItem.setTenantId(order.getTenantId());
        billItem.setProjectId(order.getProjectId());
        billItem.setBusinessId(order.getBusinessId());
        billItem.setServiceCode(item.getServiceCode());
        billItem.setInstance(sn);
        billItemRepository.save(billItem);
        oldMoney = oldMoney.add(billMoney);
        bill.setMoney(oldMoney.setScale(3, BigDecimal.ROUND_HALF_UP));
        billRepository.update(bill);

        accountAction(order, bill, billMoney);
    }

    /**
     * 生成退订账单(后付费)
     *
     * @param itemSn
     */
    private void buildCancelationBill(String itemSn) {
        ChargeItem chargeItem = chargeItemRepository.query(itemSn);
        if (null == chargeItem) {
            log.error("can not fund charge item");
            return;
        }
        OrderItem item = orderItemRepository.query(chargeItem.getItemId());
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        // 查找当月账单
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
        if (null == bill) {
            bill =
                    new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                            order.getTenantId(), month, new Date());
            bill.setUserId(order.getUserId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            bill.setMoney(BigDecimal.ZERO);
            billRepository.save(bill);
        }
        BigDecimal billMoney = bill.getMoney();
        BigDecimal finalPrice = chargeItem.getPrice();
        if (null == finalPrice) {
            finalPrice = BigDecimal.valueOf(0);
        }
        // 当前跑批账单开始时间
        Date currBillTime = getBillStartDate();
        // 跑批开始时间（资源创建时间）
        Date chargeBegin = chargeItem.getGmtCreate();
        // 生成账单时间
        Date discardDate = new Date();
        long mintues;
        BillItem billItem = new BillItem();
        // 如果资源创建时间大于跑批账单开始时间，说明还未进行过跑批计费
        if (chargeBegin.getTime() >= currBillTime.getTime()) {
            mintues = ((discardDate.getTime() - chargeBegin.getTime()) / 60000);
            billItem.setBillStart(chargeBegin);
        } else {
            mintues = ((discardDate.getTime() - currBillTime.getTime()) / 60000);
            billItem.setBillStart(currBillTime);
        }
        billItem.setBillEnd(discardDate);
        BigDecimal itemMoney =
                finalPrice.divide(new BigDecimal(60), 20, RoundingMode.HALF_UP).multiply(new BigDecimal(mintues));
        billItem.setItemId(chargeItem.getItemId());
        billItem.setSkuId(item.getSkuId());
        billItem.setChargeMode(order.getChargeMode());
        billItem.setPrice(itemMoney);
        billItem.setMoney(itemMoney);
        billItem.setGmtCreate(new Date());
        billItem.setCatalog(BillItem.Catalog.standard);
        OrderItemUtils.elementNames(item);
        billItem.setResource(JSONObject.toJSONString(item.getElementList()));
        billItem.setBillId(bill.getId());
        billItem.setUserId(order.getUserId());
        billItem.setTenantId(order.getTenantId());
        billItem.setProjectId(order.getProjectId());
        billItem.setBusinessId(order.getBusinessId());
        billItem.setServiceCode(item.getServiceCode());
        billItem.setInstance(chargeItem.getItemSn());
        billItemRepository.save(billItem);
        billMoney = billMoney.add(billItem.getMoney());
        bill.setMoney(billMoney);
        billRepository.update(bill);

        accountAction(order, bill, itemMoney);
    }

    /**
     * 租户账户相关金额处理
     *
     * @param order
     * @param bill
     * @param money
     */
    private void accountAction(ServiceOrder order, ServiceBill bill, BigDecimal money) {
        RequestContext requestContext =
                new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(), null);
        Long tenantId = order.getTenantId();
        if (null != order.getUserId() && 0L != order.getUserId()) {
            GeneralResult<User> userResult = smsUserService.detail(order.getUserId());
            if (userResult.isSuccess() && null != userResult.getData()) {
                User user = userResult.getData();
                tenantId = user.getTenantId();
                requestContext.setTarget(user.getId());
                requestContext.setTenant(user.getTenantId());
                requestContext.setCatalog(Catalog.User);
            }
        }
        if (null != tenantId && 0L != tenantId) {
            Result result = accountService
                    .accountAction(tenantId, bill.getId(), TenantAccount.Action.Consume, money, requestContext);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            bill.setStatus(ChargeCommon.BillStatus.paid);
            billRepository.update(bill);
        }
    }

    /**
     * 生成退订账单(预付费)
     * Deprecated 预付费（包年包月）退订时不创建账单，不退费用 edit by zhangyf 2022-03-23
     *
     * @param itemSn
     */
    @Deprecated
    private void buildCancelationBillOfPre(String itemSn) {
        OrderItem item = orderItemRepository.queryBySn(itemSn);
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        // 查找当月账单
        String month = new SimpleDateFormat("yyyy-MM").format(new Date());
        ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
        if (null == bill) {
            bill = new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid, null,
                    order.getTenantId(), month, new Date());
            bill.setUserId(order.getUserId());
            bill.setProjectId(order.getProjectId());
            bill.setBusinessId(order.getBusinessId());
            bill.setMoney(BigDecimal.ZERO);
            billRepository.save(bill);
        }
        BigDecimal billMoney = bill.getMoney();
        BigDecimal finalPrice = item.getFinalPrice();
        if (null == finalPrice) {
            finalPrice = BigDecimal.valueOf(0);
        }
        BillItem billItem = new BillItem();
        Date currentTime = new Date();
        long mintues = ((currentTime.getTime() - item.getExpiredTime().getTime()) / 60000);
        billItem.setBillStart(currentTime);
        billItem.setBillEnd(item.getExpiredTime());
        BigDecimal itemMoney = finalPrice.divide(BigDecimal.valueOf(31L * 24L * 60L), 20, RoundingMode.HALF_UP).multiply(new BigDecimal(mintues));
        billItem.setItemId(item.getId());
        billItem.setSkuId(item.getSkuId());
        billItem.setChargeMode(order.getChargeMode());
        billItem.setPrice(itemMoney);
        billItem.setMoney(itemMoney);
        billItem.setGmtCreate(currentTime);
        billItem.setCatalog(BillItem.Catalog.unsubscribe);
        OrderItemUtils.elementNames(item);
        billItem.setResource(JSONObject.toJSONString(item.getElementList()));
        billItem.setBillId(bill.getId());
        billItem.setUserId(order.getUserId());
        billItem.setTenantId(order.getTenantId());
        billItem.setProjectId(order.getProjectId());
        billItem.setBusinessId(order.getBusinessId());
        billItem.setServiceCode(order.getServiceCode());
        billItem.setInstance(item.getSn());
        billItemRepository.save(billItem);
        billMoney = billMoney.add(billItem.getMoney());
        bill.setMoney(billMoney);
        billRepository.update(bill);

        accountAction(order, bill, itemMoney);
    }

    /**
     * 发送消息
     *
     * @param message
     * @param operation
     * @param serviceCode
     * @param sn
     */
    private void sendMessage(OrderMessage message, Operation operation, String serviceCode, String sn) {
        // 发送站内信
        NotifyMessage notifyMessage = new NotifyMessage(message.getContext(), getServiceName(serviceCode),
                msgContent(operation, serviceCode, sn, message), true, false);
        amqpTemplate.convertAndSend(NOTICE_QUEUE, notifyMessage);
    }

    /**
     * 消息内容
     *
     * @param operation
     * @param sn
     * @return
     */
    private static final OrderItemStatus[] ORDER_ITEM_STATUSES =
            {OrderItemStatus.ChangeFailure, OrderItemStatus.OpenedFailure, OrderItemStatus.UnsubscribeFailure,
                    OrderItemStatus.ExtensionFailure};

    private String msgContent(Operation operation, String serviceCode, String sn, OrderMessage message) {
        // sonar问题: format 内部线程不稳定，采用局部变量
        SimpleDateFormat SIMPLE_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String msgResult = operationRes(message.getStatus());
        String msg = "";
        try {
            switch (operation) {
                case Application:
                    msg = "尊敬的客户，您好！您的" + getServiceName(serviceCode) + "于" + SIMPLE_DATE_FORMAT.format(new Date())
                            + msgResult + "(订单项编号(UUID): " + sn + ")，请登录控制台查看及管理。";
                    break;
                case Upgrade:
                case ScaleIn:
                case ScaleOut:
                    msg = "尊敬的客户，您好！您的" + getServiceName(serviceCode) + "于" + SIMPLE_DATE_FORMAT.format(new Date())
                            + msgResult + "(订单项编号(UUID): " + sn + ")，请登录控制台查看及管理。";
                    break;
                case Extension:
                    msg = "尊敬的客户，您好！您的" + getServiceName(serviceCode) + "于" + SIMPLE_DATE_FORMAT.format(new Date())
                            + msgResult + "(订单项编号(UUID): " + sn + ")，请登录控制台查看及管理。";
                    break;
                case Cancelation:
                    msgResult = message.getMessage();
                    msg = "尊敬的客户，您好！您的" + getServiceName(serviceCode) + "于" + SIMPLE_DATE_FORMAT.format(new Date())
                            + msgResult + "(订单项编号(UUID): " + sn + ")，请登录控制台查看及管理。";
                    break;
                default:
                    break;
            }
            OrderItemStatus status = message.getStatus();
            if (Arrays.asList(ORDER_ITEM_STATUSES).contains(status)) {
                msg += " 报错信息：\n " + message.getMessage();
            }
            return msg;
        } catch (Exception e) {
            log.error("Get error message：", e);
            return msg;
        }
    }

    private String operationRes(OrderItemStatus status) {
        String msgResult = "";
        if (OrderItemStatus.OpenedSuccess == status) {
            msgResult = "订购成功";
        } else if (OrderItemStatus.OpenedFailure == status) {
            msgResult = "订购失败";
        } else if (ChangeSuccess == status) {
            msgResult = "变更成功";
        } else if (ChangeFailure == status) {
            msgResult = "变更失败";
        } else if (OrderItemStatus.UnsubscribeSuccess == status) {
            msgResult = "退订成功";
        } else if (OrderItemStatus.UnsubscribeFailure == status) {
            msgResult = "退订失败";
        } else if (OrderItemStatus.ExtensionSuccess == status) {
            msgResult = "延期成功";
        } else if (OrderItemStatus.ExtensionFailure == status) {
            msgResult = "延期失败";
        }
        return msgResult;
    }

    private String getServiceName(String serviceCode) {
        String name;
        CloudService service = serviceRepository.queryByCode(serviceCode);
        if (null == service) {
            name = "未知";
        } else {
            name = service.getName();
        }
        return name;
    }

    /**
     * 当前小时开始时间
     *
     * @return
     */
    private Date getBillStartDate() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

}
