package com.bocloud.cos.repository;

import com.bocloud.cos.entity.CloudService;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 云服务数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Slf4j
@Repository
public class CloudServiceRepository extends JdbcGenericDao<CloudService, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    @Autowired
    public CloudServiceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 分页查询服务列表
     *
     * @param pager    分页
     * @param tenantId 租户id
     * @param isAll    是否全部查询
     * @param plugins  license授权的服务
     * @param globals   license全部的服务
     * @return
     */
    public List<CloudService> list(Pager pager, Long tenantId, Boolean isAll, List<String> plugins, List<String> globals) {
        String sql;
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        if (null == tenantId) {
            sql = "select a.*, i.icon icon, c.name catalog_name, c.has_category has_category from cloud_service a "
                + " left join " + cmpDb + ".icon i on a.icon_id = i.id left join service_catalog c on a.catalog_id = c.id "
                + " where a.is_deleted = 0 ";
        } else {
            sql = " select a.*, i.icon icon, c.name catalog_name, c.has_category has_category from cloud_service a "
                + " left join " + cmpDb + ".icon i on a.icon_id = i.id left join service_catalog c on a.catalog_id = c.id "
                + " left join service_tenant t on a.id = t.service_id  "
                + " where a.is_deleted = 0 and  t.tenant_id = " + tenantId;
        }
        if (!isAll) {
            sql += " and (a.plugin_code in (:plugins) or a.plugin_code  not in (:globals) or a.plugin_code = '' or a.plugin_code is NULL) ";
            paramMap.put("plugins", plugins);
            paramMap.put("globals", globals);
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CloudService.class, sql, paramMap);
    }

    /**
     * 按状态查询服务
     *
     * @param status  状态
     * @param local   license授权的服务
     * @param globals license全部的服务
     * @return 服务列表
     */
    public List<CloudService> listByStatus(String status, List<String> local, List<String> globals) {
        String sql = "select a.*,sc.name catalog_name,sc.has_category has_category from cloud_service a " +
                " left join service_catalog sc on  a.catalog_id=sc.id where a.is_deleted = 0 " +
                " and (plugin_code in (:pluginCode) or plugin_code not in (:globals) or plugin_code = '' or plugin_code is NULL)";
        Map<String, Object> paramMap = new HashMap<>();
        if (StringUtils.isNotEmpty(status)) {
            sql += " and a.status = :status";
            paramMap.put("status", status);
        }
        paramMap.put("pluginCode", local);
        paramMap.put("globals", globals);
        return this.list(CloudService.class, sql, paramMap);
    }

    /**
     * 查询租户授权的服务
     *
     * @param tenantId 租户id
     * @return
     */
    public List<CloudService> listTenantService(Long tenantId) {
        String sql = "Select cs.*,sc.name catalog_name, i.icon icon from cloud_service cs left join service_tenant st on cs.id = st.service_id "
            + "left join service_catalog sc on cs.catalog_id = sc.id left join " + cmpDb + ".icon i on cs.icon_id = i.id "
            + "where cs.is_deleted = 0 and  st.tenant_id = :tenantId ";
        Map<String, Object> paramsMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(CloudService.class, sql, paramsMap);
    }

    /**
     * 查询服务目录
     *
     * @param params 查询参数
     * @param sorter 排序参数
     * @return
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from cloud_service a where a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        List<CloudService> cloudServices = this.list(CloudService.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (CloudService cloudService : cloudServices) {
            SimpleBean bean = new SimpleBean(cloudService.getId(), cloudService.getName());
            beans.add(bean);
        }
        return beans;
    }

    /**
     * 数
     * 查询服务数量
     *
     * @param params   查询参数
     * @param tenantId 租户id
     * @param isAll    是否全部查询
     * @param plugins  license授权的服务
     * @param globals   license全部的服务
     * @return int
     */
    public int count(List<Param> params, Long tenantId, Boolean isAll, List<String> plugins, List<String> globals) {
        String sql;
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        if (null == tenantId) {
            sql = "select count(1) from cloud_service a where a.is_deleted = 0";
        } else {
            sql = "select count(1) from cloud_service a,service_tenant b where a.is_deleted = 0 and a.id = b.service_id and b.tenant_id = :tenantId";
            paramMap.put("tenantId", tenantId);
        }
        if (!isAll) {
            sql += " and (a.plugin_code in (:plugins) or a.plugin_code  not in (:globals) or a.plugin_code = '' or a.plugin_code is NULL) ";
            paramMap.put("plugins", plugins);
            paramMap.put("globals", globals);
        }
        sql = getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 根据查询服务详情
     *
     * @param id 服务id
     * @return
     */
    public CloudService query(Long id) {
        String sql = "select a.*,i.icon icon, c.name catalog_name,c.has_category has_category from cloud_service a "
            + " left join " + cmpDb + ".icon i on a.icon_id = i.id " + " left join service_catalog c on a.catalog_id = c.id "
            + " where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据服务编码查询服务详情
     *
     * @param code 服务编码
     * @return
     */
    public CloudService queryByCode(String code) {
        String sql = "select a.*,i.icon icon, c.name catalog_name,c.has_category has_category, p.path path from cloud_service a " +
            " left join " + cmpDb + ".icon i on a.icon_id = i.id " +
            " left join service_catalog c on a.catalog_id = c.id left join " + cmpDb + ".permission p on locate(a.code, p.service_codes) and p.is_deleted = 0 " +
            " where a.is_deleted = 0 and a.code = :code";
        Map<String, Object> params = MapTools.simpleMap(Common.CODE, code);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据菜单id查询服务数量（menu字段已废弃）
     *
     * @param menuId 菜单id
     * @return
     */
    public int count(Long menuId) {
        String sql = "select count(1) from cloud_service a where a.is_deleted = 0 and a.menu_id =:menuId";
        Map<String, Object> params = MapTools.simpleMap("menuId", menuId);
        return this.countQuery(sql, params).intValue();
    }

    /**
     * 判断服务名称是否存在
     *
     * @param name 服务名称
     * @return
     */
    public boolean existsName(String name) {
        String sql = "select a.* from cloud_service a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断服务编码是否存在
     *
     * @param code 服务编码
     * @return
     */
    public boolean existsCode(String code) {
        String sql = "select a.* from cloud_service a where a.is_deleted = 0 and a.code = :code";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断服务目录下产品编码是否存在
     *
     * @param sn        产品编码
     * @param catalogId 服务目录id
     * @return
     */
    public boolean existsSn(String sn, Long catalogId) {
        String sql = "select a.* from cloud_service a where a.is_deleted = 0 and a.sn = :sn and a.catalog_id = :catalogId";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        params.put("catalogId", catalogId);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 解除租户服务授权
     *
     * @param tenantId  租户id
     * @param serviceId 服务id
     */
    public void deleteTenant(Long tenantId, Long serviceId) {
        String sql = "delete from service_tenant where 1=1 ";
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId) {
            sql += "and tenant_id = :tenantId ";
            params.put("tenantId", tenantId);
        }
        if (null != serviceId) {
            sql += "and service_id = :serviceId ";
            params.put("serviceId", serviceId);
        }
        // 避免传入两个null而删除所有数据
        if (null == tenantId && null == serviceId) {
            sql += " and 0 = 1 ";
        }
        super.execute(sql, params);
    }

    /**
     * 解除租户所有的服务授权
     *
     * @param tenantId 租户id
     */
    public void deleteByTenant(Long tenantId) {
        String sql = "delete from service_tenant where tenant_id = :tenantId ";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        super.execute(sql, params);
    }

    public List<CloudService> listByCode(List<String> codes) {
        String sql = "select * from cloud_service where code in (:codes)";
        Map<String, Object> params = MapTools.simpleMap("codes", codes);
        return this.list(CloudService.class, sql, params);
    }

    public Integer getMaxId() {
        String sql = "select max(a.id) from cloud_service a";
        Long count = this.countQuery(sql, null);
        return count == null ? 0 : count.intValue();
    }

    public List<CloudService> queryByCatalog(List<String> codeArray) {
        String sql = " select a.* from cloud_service a where a.catalog_id in ( select id from service_catalog c " +
            " where c.code in ( :codeArray ) ) ";
        Map<String, Object> params = MapTools.simpleMap("codeArray", codeArray);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? new ArrayList<>() : list;
    }

    public List<CloudService> listByIds(List<Long> ids) {
        String sql = "select a.*,b.name catalog_name from cloud_service a left join service_catalog b on a.catalog_id = b.id " +
            " where a.is_deleted = 0 and a.id in (:ids) ";
        Map<String, Object> paramsMap = MapTools.simpleMap("ids", ids);
        return this.list(CloudService.class, sql, paramsMap);
    }

    public Boolean getByCode(String pluginCode) {
        String sql = "SELECT * from " + cmpDb + ".plugin p where p.is_deleted = 0 and p.code = :pluginCode";
        Map<String, Object> paramMap = MapTools.simpleMap("pluginCode", pluginCode);
        List<Map<String, Object>> list = this.list(sql, paramMap);
        return list.size() > 0;
    }

    public void updatePluginCode(String pluginCode) {
        String sql = "update cloud_service set plugin_code = NULL where plugin_code = (:pluginCode) ";
        Map<String, Object> param = MapTools.simpleMap("pluginCode", pluginCode);
        this.execute(sql, param);
    }
}
