package com.bocloud.cos.repository;

import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.MapTools;
import com.bocloud.cos.entity.ResourceQuota;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 租户配额数据访问
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月4日
 */
@Repository
public class ResourceQuotaRepository extends BasicGenericDao<ResourceQuota, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public ResourceQuotaRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 删除配额
     *
     * @param codes
     * @return
     * @throws DataBaseException
     */
    public boolean delete(List<String> codes) throws DataBaseException {
        String sql = "delete from resource_quota where code in (:codes)";
        Map<String, Object> paramMap = MapTools.simpleMap("codes", codes);
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 删除账号下所有配额
     *
     * @param tenantId 租户id
     * @return
     * @throws DataBaseException
     */
    public boolean delete(Long tenantId) throws DataBaseException {
        String sql = "delete from resource_quota where target=:tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 删除账号下指定类型配额
     *
     * @param target   账号id
     * @param category 配额类型
     * @return
     * @throws DataBaseException
     */
    public boolean delete(Long target, String category, String code) throws DataBaseException {
        String sql = "delete from resource_quota where 1=1 ";
        Map<String, Object> paramMap = new HashMap<>();
        if (null != target && 0L != target) {
            sql += "and target = :target ";
            paramMap.put("target", target);
        }
        if (null != category) {
            sql += " and category = :category ";
            paramMap.put("category", category);
        }
        if (null != code) {
            sql += " and code = :code ";
            paramMap.put("code", code);
        }
        if (null == category && null == code && (null == target || 0L == target)) {
            sql += " and 1=2";
        }
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 删除目标账号的配额
     *
     * @param target    账号id
     * @param category  配额类型
     * @param serviceId 服务id
     * @return
     * @throws DataBaseException
     */
    public boolean delete(Long target, String category, Long serviceId) throws DataBaseException {
        String sql = "delete from resource_quota where target = :target and category = :category and code in " +
                "(select cs.code from  cloud_service cs where is_deleted = 0 and  cs.id = :serviceId)";
        Map<String, Object> paramMap = MapTools.simpleMap("target", target);
        paramMap.put("category", category);
        paramMap.put("serviceId", serviceId);
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 查询账号配额列表
     *
     * @param tenantId 账号id
     * @return
     * @throws DataBaseException
     */
    public List<ResourceQuota> listByTenantId(Long tenantId) throws DataBaseException {
        String sql = "select * from resource_quota where target = :target";
        Map<String, Object> paramMap = MapTools.simpleMap("target", tenantId);
        return this.list(ResourceQuota.class, sql, paramMap);
    }

    /**
     * 查询账号指定类型配额列表
     *
     * @param target   账号id
     * @param category 配额类型
     * @param code     配额编码
     * @return
     * @throws DataBaseException
     */
    public List<ResourceQuota> list(Long target, String category, String code) throws DataBaseException {
        String sql = "select * from resource_quota where 1=1 ";
        Map<String, Object> paramMap = new HashMap<>(4);
        if (null != target && 0L != target) {
            sql += "and target = :target ";
            paramMap.put("target", target);
        }
        if (null != category) {
            sql += " and category = :category ";
            paramMap.put("category", category);
        }
        if (null != code) {
            sql += " and code = :code ";
            paramMap.put("code", code);
        }
        return this.list(ResourceQuota.class, sql, paramMap);
    }

    /**
     * 查询配额详情
     *
     * @param target 账号id
     * @param code   配额编码
     * @return
     * @throws DataBaseException
     */
    public ResourceQuota query(Long target, String code) throws DataBaseException {
        String sql = "select * from resource_quota where target = :target and code = :code";
        Map<String, Object> paramMap = MapTools.simpleMap("target", target);
        paramMap.put("code", code);
        List<ResourceQuota> list = this.list(ResourceQuota.class, sql, paramMap);
        return list.size() > 0 ? list.get(0) : null;
    }

    /**
     * 查询前五资源配额信息
     *
     * @param code
     * @return
     * @throws DataBaseException
     */
    public List<ResourceQuota> quotaTop5(String code) throws DataBaseException {
        String sql = "select a.*,t.name tenant_name from resource_quota a left join " + cmpDb + ".tenant t on a.target = t.id  where a.code = :code ORDER BY a.quota desc limit 5 ";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        return this.list(ResourceQuota.class, sql, paramMap);
    }

    /**
     * 查询配额详情
     *
     * @param target    账号id
     * @param serviceId 服务id
     * @return
     * @throws DataBaseException
     */
    public ResourceQuota query(Long target, Long serviceId) throws DataBaseException {
        String sql = "select rq.* from resource_quota rq left join cloud_service cs on rq.code=cs.code where rq.target = :target and cs.id = :serviceId";
        Map<String, Object> paramMap = MapTools.simpleMap("target", target);
        paramMap.put("serviceId", serviceId);
        List<ResourceQuota> list = this.list(ResourceQuota.class, sql, paramMap);
        return list.size() > 0 ? list.get(0) : null;
    }

    /**
     * 查询指定服务的配额信息列表
     *
     * @param serviceId 服务id
     * @return
     * @throws DataBaseException
     */
    public List<ResourceQuota> listServiceQuota(Long serviceId) throws DataBaseException {
        String sql = "select rq.* from resource_quota rq left join cloud_service cs on rq.code=cs.code where  cs.id = :serviceId ";
        Map<String, Object> paramMap = MapTools.simpleMap("serviceId", serviceId);
        return this.list(ResourceQuota.class, sql, paramMap);
    }

}
