package com.bocloud.cos.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.driver.ButlerRepository;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.enums.CloudProvider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.repository.VendorAccountRepository;
import com.megatron.framework.registry.RegistryClient;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.recipes.cache.CuratorCache;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 云服务供应商注册器
 */
@RequiredArgsConstructor
@Slf4j
@Component
public class ProviderRegister implements InitializingBean {

    private final VendorAccountRepository vendorAccountRepository;
    private final CuratorFramework curatorFramework;
    private final RegistryClient registryClient;

    private static final String butlerZkPath = "/cos/butler";

    public void updateProvider(List<CloudVendorAccount> providers) throws Exception {
        Map<String, String> map = convertProviders(providers);
        String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
        registryClient.put(butlerZkPath, data);
    }

    private Map<String, String> convertProviders(List<CloudVendorAccount> providers) {
        Map<String, String> map = new HashMap<>();
        Butler butler;
        for (CloudVendorAccount provider : providers) {
            String str = StringEscapeUtils.unescapeJava(provider.getAuthentication());
            JSONObject authentication = JSONObject.parseObject(str);
            String regions = authentication.getString("regions") == null ? "" : authentication.getString("regions");
            butler = new Butler(provider.getVendorName(), provider.getUuid(), authentication.getString("account"),
                    authentication.getString("password"), authentication.getString("accessKey"),
                    authentication.getString("secretKey"), regions, authentication.getString("tenant"),
                    authentication.getString("address"), provider.getVendorType(), authentication.getString("domainName"),
                    authentication.getString("projectName"), null, authentication.getString("regionId"));
            if (CloudProvider.KUBERNETES.name().equalsIgnoreCase(provider.getVendorType())) {
                butler.setToken(authentication.getString("token"));
            } else if (CloudProvider.FUSIONCLOUD.name().equalsIgnoreCase(provider.getVendorType())
                    || CloudProvider.MANAGEONE.name().equalsIgnoreCase(provider.getVendorType())
                    || CloudProvider.TIANYI.name().equalsIgnoreCase(provider.getVendorType())) {
                butler.setProjectId(authentication.getString("projectId"));
            } else if (CloudProvider.AZURE.name().equalsIgnoreCase(provider.getVendorType())) {
                butler.setSubscriptionId(authentication.getString("subscriptionId"));
                butler.setClientId(authentication.getString("clientId"));
                butler.setVersion(authentication.getString("version"));
            } else if (CloudProvider.AWS.name().equalsIgnoreCase(provider.getVendorType())) {
                butler.setRegionId("cn-north-1");
            }
            map.put(provider.getUuid(), butler.toString());
        }
        return map;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 服务启动时将数据库中的云服务供应商信息注册到ZooKeeper中
        log.info("服务启动时将数据库中的云平台账号信息注册到ZooKeeper中");
        registryClient.put(butlerZkPath, null);
        List<CloudVendorAccount> providers = vendorAccountRepository.listById(null);
        Map<String, String> map = convertProviders(providers);
        String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
        registryClient.put(butlerZkPath, data);
        ButlerRepository.getInstance().update(data);
        log.info("云平台账号信息：{}", data);
        CuratorCache curatorCache = CuratorCache.build(curatorFramework, this.registryClient.getService().getHome() + "/cos/butler");
        curatorCache.listenable().addListener((type, childData, childData1) -> {
            log.info("Cloud Service Provider data changed,Refresh them in cache!");
            ButlerRepository.getInstance().update(new String(childData1.getData(), StandardCharsets.UTF_8));
            log.info("Refresh Cloud Service Provider Success!");
        });
        curatorCache.start();
    }

}
