package com.bocloud.cos.provider.azure;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.GeneralResult;
import com.bocloud.cos.driver.BillingDriver;
import com.bocloud.cos.entity.CloudBill;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.model.AzureCloudBillModel;
import com.bocloud.cos.model.BillPeriodModel;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.repository.CloudBillRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AzureBillUtils {

    @Autowired
    private BillingDriver billingDriver;
    @Autowired
    private CloudBillRepository cloudBillRepository;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;

    public AzureBillUtils() {
    }

    @Transactional(rollbackFor = Throwable.class)
    public void azureMonthBill(List<CloudVendorAccount> vendors, String month) {
        taskExecutor.execute(() -> {
            try {
                for (CloudVendorAccount vendor : vendors) {
                    this.syncTenantMonthBill(vendor, month);
                }
            } catch (Exception e) {
                log.error("账单同步失败", e);
            }
        });
    }

    @Transactional(rollbackFor = Throwable.class)
    public void syncTenantMonthBill(CloudVendorAccount vendor, String month) {
        log.info("执行平台:{}->月份:{}->获取计费周期开始", vendor.getVendorName(), month);
        GeneralResult billPeriodsResult = billingDriver.billPeriods(vendor.getUuid());
        if (billPeriodsResult.isFailed()) {
            log.info("获取计费周期失败，失败信息:" + billPeriodsResult.getMessage());
            throw new RuntimeException("获取计费周期失败，失败信息:" + billPeriodsResult.getMessage());
        }
        List<BillPeriodModel> billPeriodModels = JSONArray.parseArray(JSONObject.toJSONString(billPeriodsResult.getData()), BillPeriodModel.class);
        log.info("执行平台:{}->月份:{}->获取计费周期结束!->数据量:{}", vendor.getVendorName(), month, billPeriodModels.size());
        Map<String, BillPeriodModel> periodModelMap = billPeriodModels.stream().collect(Collectors.toMap(periodModel -> periodModel.getName(), periodModel -> periodModel));
        String periodMonth = month.replaceAll("-", "");
        if (!periodModelMap.containsKey(periodMonth)) {
            log.info("不包含该月份的计费周期");
            throw new RuntimeException("不包含该月份的计费周期");
        }
        log.info("执行平台:{}->月份:{}->首次获取账单开始", vendor.getVendorName(), month);
        GeneralResult periodPricesResult = billingDriver.periodPrices(vendor.getUuid(), periodModelMap.get(periodMonth).getUuid());
        if (periodPricesResult.isFailed()) {
            log.info("首次获取账单失败，失败信息:" + periodPricesResult.getMessage());
            throw new RuntimeException("首次获取账单失败，失败信息:" + periodPricesResult.getMessage());
        }
        AzureCloudBillModel azureCloudBillModel = JSONObject.parseObject(JSONObject.toJSONString(periodPricesResult.getData()), AzureCloudBillModel.class);
        log.info("执行平台:{}->月份:{}->首次获取账单结束！数据量:{}", vendor.getVendorName(), month, azureCloudBillModel.getModels().size());

        log.info("执行平台:{}->月份:{}->删除本地数据开始", vendor.getVendorName(), month);
        cloudBillRepository.removeByVid(vendor.getId(), month);
        log.info("执行平台:{}->月份:{}->删除本地数据结束", vendor.getVendorName(), month);

        log.info("执行平台:{}->月份:{}->首次数据入库开始", vendor.getVendorName(), month);
        this.syncBills(azureCloudBillModel.getModels(), vendor, month);
        log.info("执行平台:{}->月份:{}->首次数据入库结束", vendor.getVendorName(), month);

        String nextLink = azureCloudBillModel.getNextLink();
        while (!StringUtils.isEmpty(nextLink)) {
            log.info("执行平台:{}->月份:{}->分页账单获取开始->url:{}", vendor.getVendorName(), month, nextLink);
            GeneralResult nextLinkPricesResult = billingDriver.nextLinkPrices(vendor.getUuid(), nextLink);
            if (nextLinkPricesResult.isFailed()) {
                log.info("分页账单失败，url:{},失败信息:" + nextLinkPricesResult.getMessage(), nextLink);
                throw new RuntimeException("分页账单失败，失败信息:" + nextLinkPricesResult.getMessage());
            }
            AzureCloudBillModel nextLinkCloudBillModel = JSONObject.parseObject(JSONObject.toJSONString(nextLinkPricesResult.getData()), AzureCloudBillModel.class);
            log.info("执行平台:{}->月份:{}->分页账单获取数据量:{}", vendor.getVendorName(), month, nextLinkCloudBillModel.getModels().size());

            log.info("执行平台:{}->月份:{}->分页数据入库开始", vendor.getVendorName(), month);
            this.syncBills(nextLinkCloudBillModel.getModels(), vendor, month);
            log.info("执行平台:{}->月份:{}->分页数据入库结束", vendor.getVendorName(), month);

            nextLink = nextLinkCloudBillModel.getNextLink();
            log.info("执行平台:{}->月份:{}->分页账单获取结束", vendor.getVendorName(), month);
        }
        log.info("执行平台:{}->月份:{}->账单拉取成功！", vendor.getVendorName(), month);
    }


    private void syncBills(List<PublicCloudBillModel> billModels, CloudVendorAccount vendor, String month) {
        if (CollectionUtils.isEmpty(billModels)) {
            return;
        }
        for (PublicCloudBillModel model : billModels) {
            CloudBill cloudBill = new CloudBill();
            BeanUtils.copyProperties(model, cloudBill);
            cloudBill.setVendorId(vendor.getId());
            cloudBill.setMonth(month);
            cloudBill.setVendorType(vendor.getVendorType());
            cloudBill.setOrderSn(cloudBill.getOrderSn());
            cloudBill.setStatus("paid"); // 已支付
            cloudBill.setChargeMode("PostCharge"); // 后付费
            cloudBill.setUnitPrice(model.getUnitPrice());
            cloudBillRepository.save(cloudBill);
        }
    }
}
