package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.driver.BillingDriver;
import com.bocloud.cos.entity.CloudBill;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.enums.CloudProvider;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.intf.CloudBillService;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.azure.AzureBillUtils;
import com.bocloud.cos.repository.CloudBillRepository;
import com.bocloud.cos.repository.VendorAccountRepository;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.entity.internel.CloudServer;
import com.bocloud.cos.entity.internel.CloudVendor;
import com.bocloud.cos.entity.internel.Tenant;
import com.bocloud.cos.entity.internel.Volume;
import com.csvreader.CsvReader;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.Future;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 公有云账单
 *
 * @author admin
 */
@Slf4j
@Service
public class CloudBillServiceImpl implements CloudBillService {

    private static OperateResult.OperateCategory operateCategory = OperateResult.OperateCategory.ResourceEvent;

    private static ThreadLocal<Integer> row = new ThreadLocal<>();

    @Value("${file.down.path:/home/cmp/s3}")
    private String fileDownPath;

    @Autowired
    private CloudBillRepository cloudBillRepository;
    @Autowired
    private ImsInternalService imsInternalService;
    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private VendorAccountRepository vendorAccountRepository;
    @Autowired
    private BillingDriver billingDriver;
    @Autowired
    private AzureBillUtils azureBillUtils;
    @Autowired
    private QingCloudBillService qingCloudBillService;

    @Override
    public GeneralResult<GridBean<CloudBill>> list(Pager pager, RequestContext context) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("feeEndTime", Common.ONE);
            int total = cloudBillRepository.count(pager.getParams());
            List<CloudBill> cloudBills = cloudBillRepository.list(pager.getPage(), pager.getRows(), pager.getParams(),
                    pager.getSorter());
            GridBean gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, cloudBills);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list CloudBill failure:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public String synchronizeData(Long vendorAccountId, String month) {
        List<CloudVendorAccount> vendorAccounts = vendorAccountRepository.listById(vendorAccountId);
        if (null == vendorAccounts || vendorAccounts.isEmpty()) {
            return "同步云账单无效：没有云平台";
        }
        if (StringUtils.isBlank(month)) {
            month = DateUtil.localYearMonthFormat(LocalDateTime.now());
        }
        Map<String, List<CloudVendorAccount>> vendorsMap = vendorAccounts.stream().collect(Collectors.groupingBy(CloudVendorAccount::getVendorType));
        for (String vendorType : vendorsMap.keySet()) {
            CloudProvider cloudProvider = CloudProvider.valueOf(vendorType);
            List<CloudVendorAccount> typeVendors = vendorsMap.get(vendorType);
            switch (cloudProvider) {
                case AZURE:
                    azureBillUtils.azureMonthBill(typeVendors, month);
                    break;
                case ALIYUN:
                case HUAWEI:
                case TENCENT:
                    syncCloudBills(typeVendors, month);
                    break;
                case AWS:
                    syncAwsBills(typeVendors, month);
                    break;
                case QCLOUD:
                    syncQingCloudBill(typeVendors);
                    break;
                default:
                    break;
            }
        }
        return "同步账单数据任务已下达，正在执行...";
    }

    private void syncCloudBills(List<CloudVendorAccount> cloudVendors, String month) {
        taskExecutor.submit(() -> {
            try {
                for (CloudVendorAccount vendor : cloudVendors) {
                    String authentication = StringEscapeUtils.unescapeJava(vendor.getAuthentication());
                    JSONObject jsonObject = JSON.parseObject(authentication);
                    String regions = jsonObject.getString("regions");
                    List<String> currentRegions = JSON.parseArray(regions, String.class);
                    if (null == currentRegions || currentRegions.isEmpty()) {
                        log.error("平台`{}`获取区域信息异常，无法同步账单数据", vendor.getVendorName());
                        continue;
                    }
                    GeneralResult GeneralResult = billingDriver.billDetailList(vendor.getUuid(), month, currentRegions.get(0));
                    List<PublicCloudBillModel> billModels = (List<PublicCloudBillModel>) GeneralResult.getData();
                    if (null == billModels || billModels.isEmpty()) {
                        log.info("平台`{}`同步到{}的数据为0条", vendor.getVendorName(), month);
                        continue;
                    }
                    log.info("平台`{}`同步到{}的数据为{}条", vendor.getVendorName(), month, billModels.size());

                    // 账单数据入库
                    syncBills(billModels, vendor, month);

                    // 计算租户二次计价金额
                    if (CloudProvider.valueOf(vendor.getVendorType()) == CloudProvider.AZURE) {
                        syncTenantBill();
                    }
                }
            } catch (Exception e) {
                log.error("同步公有云账单异常：{}", e);
            }
        });
    }

    private void syncAwsBills(List<CloudVendorAccount> cloudVendors, String month) {
        taskExecutor.submit(() -> {
            try {
                for (CloudVendorAccount vendor : cloudVendors) {
                    String str = StringEscapeUtils.unescapeJava(vendor.getAuthentication());
                    JSONObject authentication = JSON.parseObject(str);
                    String regions = authentication.getString("regions");
                    List<String> currentRegions = JSON.parseArray(regions, String.class);
                    if (ListTool.isEmpty(currentRegions)) {
                        log.error("平台`{}`获取区域信息异常，无法同步账单数据", vendor.getVendorName());
                        continue;
                    }
                    GeneralResult GeneralResult = billingDriver.queryInstanceBill(vendor.getUuid(), month, fileDownPath);
                    if (GeneralResult.isFailed()) {
                        log.warn("{}:底层API获取账单失败！失败消息:{}", vendor.getVendorName(), GeneralResult.getMessage());
                        continue;
                    }
                    List<String> filePaths = (List<String>) GeneralResult.getData();
                    if (ListTool.isEmpty(filePaths)) {
                        continue;
                    }
                    for (String path : filePaths) {
                        File file = new File(path);
                        if (!file.exists()) {
                            log.info("{}:账单文件不存在,文件路径:{},账单退出同步！", vendor.getVendorName(), file.getAbsolutePath());
                            continue;
                        }
                        // 账单数据入库
                        syncBills(importAwsBill(file), vendor, month);
                    }
                }
            } catch (Exception e) {
                log.error("同步公有云账单异常：{}", e);
            }
        });
    }

    public static List<PublicCloudBillModel> importAwsBill(File file) {
        List<PublicCloudBillModel> billModels = new ArrayList<>();
        try {
            CsvReader csvReader = new CsvReader(new FileInputStream(file), StandardCharsets.UTF_8);
            csvReader.readHeaders();
            String rowData;
            String[] headers = csvReader.getHeaders();
            while (csvReader.readRecord()) {
                rowData = csvReader.getRawRecord();
                String[] split = rowData.split("\",\"");
                PublicCloudBillModel bill = new PublicCloudBillModel();
                // 加入导入账单的信息
                bill.setMoney(new BigDecimal(getColumn("BlendedCost", split, headers)));
                bill.setProductName(getColumn("ProductName", split, headers));
                bill.setProductDetail(getColumn("UsageType", split, headers));
                bill.setRecordType(getColumn("RecordType", split, headers));
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String usageStartDate = getColumn("UsageStartDate", split, headers);
                if (!"".equals(usageStartDate.trim())) {
                    bill.setFeeBeginTime(sdf.parse(usageStartDate));
                }

                String usageEndDate = getColumn("UsageEndDate", split, headers);
                if (!"".equals(usageEndDate.trim())) {
                    bill.setFeeEndTime(sdf.parse(usageEndDate));
                }

                String resource = getColumn("ResourceId", split, headers);
                bill.setResourceId(resource);
                if (null != resource && !resource.trim().isEmpty()) {
                    String[] resourceAttr = resource.split(":");
                    if (resourceAttr.length > 0) {
                        // 实例
                        bill.setResourceName(resourceAttr[resourceAttr.length - 1]);
                    } else {
                        // 磁盘
                        bill.setResourceName(resource);
                    }
                } else {
                    bill.setResourceName(null);
                }
                billModels.add(bill);
            }
            return billModels;
        } catch (Exception e) {
            log.error("导入账单失败!", e);
            return billModels;
        }
    }

    private static String getColumn(String head, String[] split, String[] headers) {
        try {
            for (int i = 0; i < headers.length; i++) {
                if (headers[i].equals(head)) {
                    return split[i];
                }
            }
            return "";
        } catch (Exception e) {
            log.error("aws getColumn error:", e);
            return "";
        }
    }

    private void syncBills(List<PublicCloudBillModel> billModels, CloudVendorAccount vendorAccount, String month) {
        if (ListTool.isEmpty(billModels)) {
            cloudBillRepository.deleteByVendor(vendorAccount.getId(), null);
            return;
        }
        for (PublicCloudBillModel model : billModels) {
            CloudBill cloudBill = cloudBillRepository.listByVendorAndOrderSn(vendorAccount.getId(), model.getOrderSn());
            if (null != cloudBill) {
                continue;
            }
            CloudBill newBill = JSON.parseObject(JSON.toJSONString(model), CloudBill.class);
            newBill.setVendorId(vendorAccount.getId());
            newBill.setMonth(StringUtils.isEmpty(model.getMonth()) ? month : model.getMonth());
            newBill.setVendorType(vendorAccount.getVendorType());
            newBill.setOrderSn(model.getOrderSn());
            newBill.setStatus("paid");
            newBill.setChargeMode("PostCharge");
            newBill.setSubscriptionId(model.getSubscriptionId());
            newBill.setConsumedService(model.getConsumedService());
            newBill.setUnitPrice(model.getUnitPrice());
            cloudBillRepository.save(newBill);
        }
    }

    private void syncQingCloudBill(List<CloudVendorAccount> vendorAccounts) {
        qingCloudBillService.syncBill(vendorAccounts);
    }

    /**
     * 同步租户账单，进行二次计价
     *
     * @throws Exception
     */
    private void syncTenantBill() {
        List<String> types = Arrays.asList(new String[]{CloudProvider.AZURE.name()});
        List<CloudVendorAccount> cloudVendors = vendorAccountRepository.listByTypes(types);
        List<CloudBill> bills = new ArrayList<>();
        for (CloudVendorAccount vendor : cloudVendors) {
            List<CloudBill> cloudBills = cloudBillRepository.listByVendor(vendor.getId(), null);
            if (!ListTool.isEmpty(cloudBills)) {
                bills.addAll(cloudBills);
            }
        }
        GeneralResult<List<Tenant>> result = smsTenantService.listAll(new ArrayList<>());
        List<CloudServer> servers = imsInternalService.listServers(RequestContext.builder().build());
        Map<String, CloudServer> serverMap = servers.stream().filter(cloudServer -> null != cloudServer.getInstanceId())
                .collect(Collectors.toMap(CloudServer::getInstanceId, Function.identity()));
        List<Volume> volumes = imsInternalService.listVms();
        Map<String, Volume> volumeMap = volumes.stream().filter(volume -> null != volume.getVolumeUuid())
                .collect(Collectors.toMap(Volume::getVolumeUuid, Function.identity()));
        if (result.isFailed()) {
            return;
        }
        List<Tenant> tenants = result.getData();
        if (ListTool.isEmpty(tenants)) {
            return;
        }
        Map<Long, BigDecimal> discountMap = tenants.stream().filter(tenant -> null != tenant.getDiscount())
                .collect(Collectors.toMap(Tenant::getId, Tenant::getDiscount));
        List<CloudBill> updateBills = new ArrayList<>();
        for (CloudBill bill : bills) {
            String resourceId = bill.getResourceId();
            if (StringUtils.isEmpty(resourceId)) {
                continue;
            }
            CloudServer server = serverMap.get(resourceId);
            if (null != server) {
                bill.setTenantId(server.getTenantId());
                bill.setProjectId(server.getProjectId());
                // 计算租户价格
                BigDecimal discount = discountMap.get(bill.getTenantId());
                if (null != discount) {
                    bill.setTenantMoney(bill.getTotalCost().multiply(discount));
                }
                updateBills.add(bill);
            } else {
                Volume volume = volumeMap.get(resourceId);
                if (null == volume) {
                    return;
                }
                bill.setTenantId(volume.getTenantId());
                bill.setProjectId(volume.getProjectId());
                // 计算租户价格
                BigDecimal discount = discountMap.get(bill.getTenantId());
                if (null != discount) {
                    bill.setTenantMoney(bill.getTotalCost().multiply(discount));
                }
                updateBills.add(bill);
            }
        }
        if (!ListTool.isEmpty(updateBills)) {
            cloudBillRepository.batchUpdate(updateBills, 1000);
        }
    }

    @Override
    public GeneralResult importBill(MultipartFile file, CloudBill cloudBill, RequestContext context) {
        try {
            Long vendorId = cloudBill.getVendorId();
            CloudVendor vendor = imsInternalService.getVendorDetail(vendorId);
            Assert.notNull(vendor, "平台验证失败");
            taskExecutor.execute(() -> {
                String operate = vendor.getType().toLowerCase() + ".bill";
                String message = "账单导入失败";
                try {
                    message = "账单导入成功";
                    CsvReader csvReader = new CsvReader(file.getInputStream(), Charset.forName("UTF-8"));
                    csvReader.readHeaders();
                    List<CloudBill> cloudBills = new ArrayList<>();
                    String str = null;// 每行数据
                    while (csvReader.readRecord()) {
                        str = csvReader.getRawRecord();// 读取一行数据
                        String[] split = str.split(",");
                        CloudBill entity = new CloudBill();
                        entity.setVendorId(vendor.getId());
                        entity.setVendorType(vendor.getType());
                        // 加入导入账单的信息
                        entity.setOrderSn(split[0]);
                        entity.setMonth(split[1]);
                        entity.setProductName(split[2]);
                        entity.setProductDetail(split[3]);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
                        entity.setFeeBeginTime(sdf.parse(split[4]));
                        entity.setFeeEndTime(sdf.parse(split[5]));
                        entity.setMoney(new BigDecimal(split[6]));
                        entity.setChargeMode("PostCharge"); // 后付费
                        entity.setStatus("paid"); // 已支付
                        cloudBills.add(entity);
                    }
                    // save 到数据库
                    cloudBills.forEach(cloudBillRepository::save);
                    resourceEventPublisher.send(new OperateResult(true, message, operate,
                            MapTools.simpleMap("vendorId", vendor.getId()), operateCategory, context));
                } catch (Exception e) {
                    log.error("Azure BILL fail:", e);
                    resourceEventPublisher.send(new OperateResult(false, message, operate,
                            MapTools.simpleMap("vendorId", vendor.getId()), operateCategory, context));
                }
            });
            return new GeneralResult<>(true, "导入任务已下发，正在执行...");
        } catch (Exception e) {
            log.error("import AzureCloudBill failure:", e);
            return new GeneralResult<>(false, "导入失败");
        }
    }

    /**
     * 创建标题
     *
     * @param sheet
     * @param style
     */
    public void createVmReport(SXSSFSheet sheet, CellStyle style) {
        // 模板标题处理
        String[] titles = new String[]{"账期", "所属平台", "流水号", "资源", "资源详情", "开始时间", "结束时间", "账单金额(元)", "计费类型", "账单状态"};
        int number = -1;
        SXSSFRow row = sheet.createRow(0);
        for (String title : titles) {
            number++;
            row.createCell(number).setCellValue(title);
            sheet.getRow(0).getCell(number).setCellStyle(style);
            sheet.setColumnWidth(number, 30 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
    }

    @Override
    public GeneralResult download(Pager pager, RequestContext context) {
        SXSSFWorkbook workbook = new SXSSFWorkbook();
        try {
            SXSSFSheet sheet = workbook.createSheet("外部账单");
            sheet.setRandomAccessWindowSize(-1);
            // 样式一，标题数据蓝色填充
            CellStyle style = workbook.createCellStyle();
            style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
            style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);
            style.setAlignment(HorizontalAlignment.CENTER);
            // 样式二，内容样式
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderTop(BorderStyle.THIN);
            cellStyle.setBorderBottom(BorderStyle.THIN);
            cellStyle.setBorderLeft(BorderStyle.THIN);
            cellStyle.setBorderRight(BorderStyle.THIN);
            cellStyle.setAlignment(HorizontalAlignment.CENTER);
            cellStyle.setVerticalAlignment(VerticalAlignment.TOP);
            // 标题写入
            createVmReport(sheet, style);
            // 读写数据
            Map<String, String> sorter = Maps.newHashMap();
            sorter.put("feeEndTime", Common.ONE);
            List<Param> pageParams = pager.getParams();
            List<CloudBill> cloudBills = cloudBillRepository.list(1, Integer.MAX_VALUE, pager.getParams(), sorter);
            Integer total = cloudBills.size();
            // 每个线程处理的数量
            int numbers = 200;
            if (total < numbers) {
                excelExporter(sheet, cellStyle, 1, new Pager(1, numbers, pageParams, sorter), context);
            } else {
                int pages = total / numbers;
                int lastNumers = total % numbers;
                int rowNumber = 1;
                List<Future<Boolean>> futures = new ArrayList<>();
                for (int i = 1; i <= pages; i++) {
                    int page = i;
                    int finalRowNumber = rowNumber;
                    futures.add(taskExecutor.submit(() -> excelExporter(sheet, cellStyle, finalRowNumber,
                            new Pager(page, numbers, pageParams, sorter), context)));
                    rowNumber += numbers;
                }
                // 获取结果
                for (Future<Boolean> future : futures) {
                    Boolean successful = future.get();
                    if (successful) {
                        // 收到写入完成提示
                        continue;
                    }
                }
                // 整页数据的异步查询写入完成，处理最后一页不足整页的数据
                if (0 != lastNumers) {
                    excelExporter(sheet, cellStyle, pages * numbers + 1,
                            new Pager(pages + 1, numbers, pageParams, sorter), context);
                }
            }
            String fileName = "账单数据统计";
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage());
            return new GeneralResult<>(false, e.getMessage());
        } finally {
            if (null != workbook) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("error message：", e);
                }
            }
        }
    }

    @Override
    public GeneralResult<List<String>> queryProducts(String vendorType) {
        List<CloudBill> cloudBills = cloudBillRepository.queryProducts(vendorType);
        if (org.apache.commons.collections4.CollectionUtils.isEmpty(cloudBills)) {
            return new GeneralResult<>(true, new ArrayList<>(), "查询成功");
        }
        List<String> products = cloudBills.stream().map(t -> t.getProductName()).collect(Collectors.toList());
        return new GeneralResult<>(true, products, "查询成功");
    }

    /**
     * excel导出
     *
     * @param sheet
     * @param cellStyle
     * @param rowNumber
     * @param pager
     * @param context
     * @return
     */
    public Boolean excelExporter(SXSSFSheet sheet, CellStyle cellStyle, int rowNumber, Pager pager,
                                 RequestContext context) {
        int number = rowNumber;
        try {
            GeneralResult result = this.list(pager, context);
            if (result.isFailed()) {
                log.error("获取账单数据失败", result.getMessage());
                return false;
            }
            List<CloudBill> cloudBills = JSON.parseObject(JSON.toJSONString(result.getData()))
                    .getJSONArray("rows").toJavaList(CloudBill.class);
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for (CloudBill cloudBill : cloudBills) {
                row.set(++number);
                // 创建数据初始化行
                SXSSFRow sheetRow = sheet.createRow(row.get());
                // "账期", "所属平台", "流水号", "资源", "资源详情", "开始时间", "结束时间",
                // "账单金额(元)", "计费类型", "账单状态"
                sheetRow.createCell(0).setCellValue(cloudBill.getMonth());
                sheet.getRow(row.get()).getCell(0).setCellStyle(cellStyle);
                sheetRow.createCell(1).setCellValue(cloudBill.getVendorName());
                sheet.getRow(row.get()).getCell(1).setCellStyle(cellStyle);
                sheetRow.createCell(2).setCellValue(cloudBill.getOrderSn());
                sheet.getRow(row.get()).getCell(2).setCellStyle(cellStyle);
                sheetRow.createCell(3).setCellValue(cloudBill.getProductName());
                sheet.getRow(row.get()).getCell(3).setCellStyle(cellStyle);
                sheetRow.createCell(4).setCellValue(cloudBill.getProductDetail());
                sheet.getRow(row.get()).getCell(4).setCellStyle(cellStyle);
                sheetRow.createCell(5).setCellValue(null == cloudBill.getFeeBeginTime() ? "" : simpleDateFormat.format(cloudBill.getFeeBeginTime()));
                sheet.getRow(row.get()).getCell(5).setCellStyle(cellStyle);
                sheetRow.createCell(6).setCellValue(null == cloudBill.getFeeEndTime() ? "" : simpleDateFormat.format(cloudBill.getFeeEndTime()));
                sheet.getRow(row.get()).getCell(6).setCellStyle(cellStyle);
                sheetRow.createCell(7).setCellValue(cloudBill.getMoney().doubleValue());
                sheet.getRow(row.get()).getCell(7).setCellStyle(cellStyle);
                String chargeMode = cloudBill.getChargeMode();
                if ("PostCharge".equalsIgnoreCase(chargeMode)) {
                    chargeMode = "后付费";
                } else if ("PreCharge".equalsIgnoreCase(chargeMode)) {
                    chargeMode = "预付费";
                } else {
                    chargeMode = "未知";
                }
                sheetRow.createCell(8).setCellValue(chargeMode);
                sheet.getRow(row.get()).getCell(8).setCellStyle(cellStyle);
                String status = cloudBill.getStatus();
                if ("paid".equalsIgnoreCase(status)) {
                    status = "已支付";
                } else {
                    status = "未知";
                }
                sheetRow.createCell(9).setCellValue(status);
                sheet.getRow(row.get()).getCell(9).setCellStyle(cellStyle);
            }
            return true;
        } catch (Exception e) {
            log.error("账单数据【" + row.get() + "行至" + (row.get() + 20) + "行】写入失败", e);
            return false;
        } finally {
            row.remove();
        }
    }
}