package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.service.internal.SmsInternalService;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.model.BatchServiceFlowBean;
import com.bocloud.cos.model.ServiceFlowBean;
import com.bocloud.cos.repository.*;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 云服务业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class CloudServiceServiceImpl implements CloudServiceService {

    private final ServiceCatalogRepository serviceCatalogRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceFlowConfigRepository flowConfigRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final ResourceQuotaRepository quotaRepository;
    private final SmsInternalService smsInternalService;
    private final OrderSnRepository snRepository;
    private final LockFactory lockFactory;
    private final ServiceTenantRepository serviceTenantRepository;
    private final CurrentService currentService;

    @Override
    public GeneralResult<GridBean<CloudService>> list(Pager pager, Boolean isAll) {
        pager.getSorter().put("gmtCreate", Common.ONE);
        Long paramTenantId = null;
        for (Param pm : pager.getParams()) {
            Map<String, Object> param = pm.getParam();
            Object obj = param.get("tenantId");
            if (null == obj) {
                continue;
            }
            paramTenantId = Long.valueOf(obj.toString());
            param.remove("tenantId");
            break;
        }
        License content = LicenseDaemon.instance(currentService).content();
        List<String> plugins = content.getPlugins();
        List<String> global = content.getGlobals();
        int total = this.cloudServiceRepository.count(pager.getParams(), paramTenantId, isAll, plugins, global);
        List<CloudService> list = this.cloudServiceRepository.list(pager, paramTenantId, isAll, plugins, global);
        for (CloudService cloudService : list) {
            List<ServiceFlowBean> flowList = new ArrayList<>();
            Map<ApplyCategory, ServiceFlowConfig> flowConfigMap =
                    flowConfigRepository.getServiceFlowConfig(cloudService.getId()).stream()
                            .collect(Collectors.toMap(ServiceFlowConfig::getCategory, Function.identity()));
            flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ApplicationOperate) != null ?
                    flowConfigMap.get(ApplyCategory.ApplicationOperate).getFlowId() : null,
                    ApplyCategory.ApplicationOperate));
            flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.AlterationOperate) != null ?
                    flowConfigMap.get(ApplyCategory.AlterationOperate).getFlowId() : null,
                    ApplyCategory.AlterationOperate));
            flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ExtensionOperate) != null ?
                    flowConfigMap.get(ApplyCategory.ExtensionOperate).getFlowId() : null,
                    ApplyCategory.ExtensionOperate));
            flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.UnsubscribeOperate) != null ?
                    flowConfigMap.get(ApplyCategory.UnsubscribeOperate).getFlowId() : null,
                    ApplyCategory.UnsubscribeOperate));
            flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ResourceComputerClone) != null ?
                    flowConfigMap.get(ApplyCategory.ResourceComputerClone).getFlowId() : null,
                    ApplyCategory.ResourceComputerClone));
            cloudService.setFlowList(flowList);
        }
        GridBean<CloudService> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @Override
    @Transactional
    public Result remove(Long id, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("serviceId", id), Sign.EQ));
            int count = serviceSkuRepository.count(params);
            Assert.isTrue(count == 0, "该服务下存在规格无法删除");
            service.setMenderId(userId);
            cloudServiceRepository.update(service);
            cloudServiceRepository.remove(CloudService.class, service.getId());
            snRepository.removeByServiceId(id);
            cloudServiceRepository.deleteTenant(null, id);
            return new Result(true, "删除成功");
        } catch (RuntimeException e) {
            throw new RuntimeException("删除失败:" + e.getMessage(), e);
        }
    }

    @Override
    public GeneralResult<CloudService> detail(Long id) {
        CloudService service = cloudServiceRepository.query(id);
        List<ServiceSla> slas = serviceSlaRepository.listByService(id, ServiceSla.Status.enable.name());
        if (!ListTool.isEmpty(slas)) {
            service.setSlaId(slas.get(0).getId());
        }
        return new GeneralResult<>(true, service, "查询成功");
    }

    @Override
    public Result create(CloudService service) {
        ServiceCatalog catalog = serviceCatalogRepository.get(ServiceCatalog.class, service.getCatalogId());
        if (null == catalog) {
            return new GeneralResult<>(false, "服务目录不存在");
        }
        boolean flag = cloudServiceRepository.existsName(service.getName());
        if (flag) {
            return new Result(false, "服务名称已存在");
        }
        flag = cloudServiceRepository.existsCode(service.getCode());
        if (flag) {
            return new Result(false, "服务编码已存在");
        }
        flag = cloudServiceRepository.existsSn(service.getSn(), service.getCatalogId());
        if (flag) {
            return new Result(false, "当前服务目录下的服务编号已存在");
        }
        flag = smsInternalService.getLicenseService(service.getVendorType());
        if (!flag) {
            return new Result(false, "未授权 " + service.getVendorType() + " License，请联系管理员!");
        }
        service.setStatus(CloudService.Status.PUBLISHED.name());
        Integer maxId = cloudServiceRepository.getMaxId();
        if (null == maxId) {
            maxId = 0;
        }
        service.setSn(String.format("%02d", maxId));
        cloudServiceRepository.save(service);
        return new Result(true, "新增成功");
    }

    @Transactional
    @Override
    public Result modify(CloudService service, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + service.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService object = cloudServiceRepository.query(service.getId());
            Assert.notNull(service, "数据不存在！");
            if (!object.getName().equals(service.getName()) && cloudServiceRepository.existsName(service.getName())) {
                return new Result(false, "服务已存在");
            }
            if (!object.getCode().equals(service.getCode()) && cloudServiceRepository.existsCode(service.getCode())) {
                return new Result(false, "服务编码已存在");
            }
            if (!object.getSn().equals(service.getSn()) && cloudServiceRepository
                    .existsSn(service.getSn(), service.getCatalogId())) {
                return new Result(false, "当前服务目录下的服务编号已存在");
            }
            object.setName(service.getName());
            object.setCode(service.getCode());
            object.setSn(service.getSn());
            object.setIconId(service.getIconId());
            object.setCatalogId(service.getCatalogId());
            object.setRemark(service.getRemark());
            object.setUnit(service.getUnit());
            object.setBillPolicy(service.getBillPolicy());
            object.setBillable(service.getBillable());
            object.setBasicPrice(service.getBasicPrice());
            object.setTemplateId(service.getTemplateId());
            object.setMenderId(userId);
            object.setGmtModify(new Date());
            boolean result = cloudServiceRepository.update(object);
            return result ? new Result(true, "修改成功") : new Result(false, "修改失败");
        } catch (RuntimeException e) {
            throw e;
        }
    }

    /**
     * 查询租户可授权的服务
     * 创建租户时 给子租户用 授权最大范围受 租户自己限制 最小值0
     * 授权给项目 给项目使用 授权最大范围受 租户自己限制 最小值=项目使用量
     * 授权本租户 给自己租户使用 授权最大范围受 父租户限制 最小值=租户自身使用量
     *
     * @param tenantId
     * @param event
     * @param quotaAble
     * @param requestContext
     * @return
     */
    @Override
    public GeneralResult<List<CloudService>> getAbleServices(Long tenantId, String event, Boolean quotaAble,
                                                             RequestContext requestContext) {
        List<CloudService> list = new ArrayList<>();
        License content = LicenseDaemon.instance(currentService).content();
        switch (event) {
            case "toCreateTenant":
                list = getTenantCreateGrantAble(requestContext, content.getPlugins(), content.getGlobals());
                break;
            case "toAssignTenant":
                if (Optional.ofNullable(quotaAble).orElse(false)) {
                    list = getServiceGrantScope(tenantId, requestContext, content.getPlugins(), content.getGlobals());
                } else {
                    list = getServiceGrantAble(tenantId, requestContext, content.getPlugins(), content.getGlobals());
                }
                break;
            default:
                break;

        }
        return new GeneralResult<>(true, list, "查询成功");
    }

    /**
     * 查询租户服务可授权范围(最大值 最小值)
     *
     * @param tenantId 租户ID
     * @param context  当前用户
     * @param local    本地已授权的插件
     * @param globals  全部插件
     * @return 服务列表
     */
    private List<CloudService> getServiceGrantScope(Long tenantId, RequestContext context, List<String> local, List<String> globals) {
        List<CloudService> list = new ArrayList<>();
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            //查询租户已经授权的服务
            Map<String, CloudService> tenantServices = cloudServiceRepository.listTenantService(tenantId).stream()
                    .collect(
                            Collectors.toMap(CloudService::getCode, Function.identity(), (service1, service2) -> service2));
            List<CloudService> allPublishService = this.cloudServiceRepository.listByStatus("PUBLISHED", local, globals);
            //查询租户授权的服务配额
            Map<String, ResourceQuota> tenantQuota =
                    quotaRepository.list(tenantId, ResourceQuota.Category.Service.name(), null).stream()
                            .collect(Collectors.toMap(ResourceQuota::getCode, Function.identity()));
            for (CloudService cloudService : allPublishService) {
                //处理授权配额最大值最小值(无单位的服务不需要配额)
                if (cloudService.getUnit() == null || cloudService.getUnit().length() <= 0) {
                    cloudService.setReleasable(true);
                    CloudService tenantService = tenantServices.get(cloudService.getCode());
                    if (tenantService != null) {
                        //已经授权的服务check改为true
                        cloudService.setCheck(true);
                    }
                } else {
                    //设置服务配额默认值 默认最大99999 最小0 默认未授权 可释放配额
                    cloudService.setMaxQuota(99999999L);
                    cloudService.setMinQuota(0L);
                    cloudService.setCheck(false);
                    cloudService.setReleasable(true);
                    //查询当前服务是否授权
                    CloudService tenantService = tenantServices.get(cloudService.getCode());
                    if (tenantService != null) {
                        //已经授权的服务check改为true
                        cloudService.setCheck(true);
                        ResourceQuota quota = tenantQuota.get(cloudService.getCode());
                        //已经授权的服务需要修改配额最大值最小值及能否释放
                        if (quota != null) {
                            cloudService.setQuota(quota.getQuota());
                            // 有配额使用的服务不可释放
                            if (quota.getUsed() > 0) {
                                // 有授权和使用得不可以释放
                                cloudService.setReleasable(false);
                                //租户服务授权最小值受分配量限制（使用量在租户分配量里面）
                                cloudService.setMinQuota(quota.getUsed());
                            }
                        }
                    }
                }

                list.add(cloudService);
            }
        }
        return list;
    }

    /**
     * 查询租户可授权服务
     *
     * @param tenantId
     * @param context
     * @return
     */
    private List<CloudService> getServiceGrantAble(Long tenantId, RequestContext context, List<String> local, List<String> globals) {
        List<CloudService> list = new ArrayList<>();
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            //查询租户已经授权的服务
            Map<String, ServiceTenant> serviceTenants = serviceTenantRepository.listByTenantId(tenantId).stream()
                    .collect(Collectors.toMap(ServiceTenant::getServiceCode, Function.identity()));
            List<CloudService> allPublishService = this.cloudServiceRepository.listByStatus("PUBLISHED", local, globals);
            for (CloudService cloudService : allPublishService) {
                //查询当前服务是否授权
                ServiceTenant serviceTenant = serviceTenants.get(cloudService.getCode());
                cloudService.setCheck(false);
                if (serviceTenant != null) {
                    //已经授权的服务check为true
                    cloudService.setCheck(true);
                }
                list.add(cloudService);
            }
        }
        return list;
    }

    /**
     * 查询租户创建服务可授权范围
     *
     * @param context
     * @return
     */
    private List<CloudService> getTenantCreateGrantAble(RequestContext context, List<String> local, List<String> globals) {
        List<CloudService> list = new ArrayList<>();
        //管理端创建已经发布的服务都能选最大值99999 最小值0
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            list = this.cloudServiceRepository.listByStatus("PUBLISHED", local, globals);
            //开启配额才需要设置最大值最小值
            for (CloudService cloudService : list) {
                //有单位的才需要设置配额最大值最小值
                if (cloudService.getUnit() == null || cloudService.getUnit().length() <= 0) {
                    continue;
                }
                // 一级租户最大配额 999999L
                cloudService.setMaxQuota(99999999L);
                cloudService.setMinQuota(0L);
            }
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = DataBaseException.class)
    public Result updateStatus(Long id, String status, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            String message = CloudService.Status.PUBLISHED.name().equalsIgnoreCase(status) ? "上架" : "下架";
            if (CloudService.Status.PUBLISHED.name().equalsIgnoreCase(status)
                    && !smsInternalService.getLicenseService(service.getVendorType())
            ) {
                // 如果上架某个服务，需要先判断服务是否有License
                return new GeneralResult<>(false, "未授权 " + service.getVendorType() + " License，请联系管理员!");
            }
            service.setStatus(status);
            service.setMenderId(userId);
            service.setGmtModify(new Date());
            boolean flag = cloudServiceRepository.update(service);
            if (!flag) {
                return new Result(false, message + "失败");
            }
            return new Result(true, message + "成功");
        } catch (Exception e) {
            log.error("update cloud service status error:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = DataBaseException.class)
    public Result updateTemplate(Long id, String templateName, RequestContext context) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            service.setTemplateName(templateName);
            service.setMenderId(context.getTarget());
            service.setGmtModify(new Date());
            boolean flag = cloudServiceRepository.update(service);
            if (!flag) {
                return new Result(false, "更新失败");
            }
            return new Result(true, "更新成功");
        } catch (Exception e) {
            log.error("Modify cloud service template error:", e);
            return new Result(false, "更新失败");
        }
    }

    @Override
    @Transactional
    public Result batchPublished(List<Long> ids, Long userId) {
        List<CloudService> cloudServices = cloudServiceRepository.listByIds(ids);
        for (CloudService cloudService : cloudServices) {
            cloudService.setStatus(CloudService.Status.PUBLISHED.name());
            cloudService.setMenderId(userId);
            cloudService.setGmtModify(new Date());
        }
        cloudServices.forEach(cloudServiceRepository::update);
        return new Result(true, "批量上架服务成功");
    }

    @Override
    @Transactional
    public Result batchUnpublished(List<Long> ids, Long userId) {
        List<CloudService> cloudServices = cloudServiceRepository.listByIds(ids);
        for (CloudService cloudService : cloudServices) {
            cloudService.setStatus(CloudService.Status.UNPUBLISHED.name());
            cloudService.setMenderId(userId);
            cloudService.setGmtModify(new Date());
        }
        cloudServices.forEach(cloudServiceRepository::update);
        return new Result(true, "批量下架服务成功");
    }

    @Override
    public GeneralResult<List<CloudService>> listByCode(List<String> codes) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listByCode(codes);
            return new GeneralResult<>(true, cloudServices, "获取服务列表成功");

        } catch (Exception e) {
            log.error("get cloud service by code error,", e);
            return new GeneralResult<>(false, "获取服务列表失败");
        }
    }

    @Override
    public GeneralResult<List<CloudService>> getTenantService(Long id, RequestContext requestContext) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(id);
            return new GeneralResult<>(true, cloudServices, "查询成功");

        } catch (Exception e) {
            log.error("get tenant's cloud service error,", e);
            return new GeneralResult<>(false, "获取租户服务列表失败");
        }
    }

    @Override
    public GeneralResult condition(String condition, RequestContext context) {
        if (StringUtils.isEmpty(condition)) {
            return new GeneralResult<>(false, "参数格式错误");
        }
        try {
            JSONObject object = JSONObject.parseObject(condition);
            if ("queryByCode".equalsIgnoreCase(object.getString("condition"))) {
                CloudService service = cloudServiceRepository.queryByCode(object.getString("code"));
                List<ServiceSla> slas =
                        serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
                if (!slas.isEmpty()) {
                    service.setSlaId(slas.get(0).getId());
                }
                return new GeneralResult<>(true, service, "查询成功");
            }
            if ("queryByCatalog".equalsIgnoreCase(object.getString("condition"))) {
                List<String> catalogArray = JSONObject.parseArray(object.getString("catalogs"), String.class);
                if (ListTool.isEmpty(catalogArray)) {
                    return new GeneralResult<>(false, "参数格式错误");
                }
                List<CloudService> services = cloudServiceRepository.queryByCatalog(catalogArray);
                return new GeneralResult<>(true, services, "查询成功");
            }
            if ("getServiceType".equalsIgnoreCase(object.getString("condition"))) {
                String catalog = object.getString("catalog");
                Set<String> serviceType = this.getServiceType(catalog, context);
                return new GeneralResult<>(true, serviceType, "查询成功");
            }
        } catch (Exception e) {
            log.error("list by condition error", e);
        }
        return new GeneralResult<>(false, "未知类型");
    }

    private Set<String> getServiceType(String catalog, RequestContext context) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(context.getTenant());
            List<CloudService> server = new ArrayList<>();
            switch (catalog) {
                case "server":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("server".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "volume":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("volume".equals(suffix) || "cbs".equals(suffix) || "disk".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "slb":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("slb".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "vpc":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("vpc".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "eip":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("eip".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                default:
                    break;
            }
            return server.stream().map(CloudService::getVendorType).collect(Collectors.toSet());
        } catch (Exception e) {
            log.error("list permission");
            return null;
        }
    }

    @Override
    public GeneralResult<List<String>> vendors(Long catalogId, Long tenantId) {
        List<CloudService> cloudServices = cloudServiceRepository.listTenantService(tenantId);
        List<String> vendorTypes =
                cloudServices.stream().filter(cloudService -> cloudService.getCatalogId().equals(catalogId))
                        .map(CloudService::getVendorType).distinct().collect(Collectors.toList());
        return new GeneralResult<>(true, vendorTypes, "查询失败");
    }

    @Override
    public GeneralResult<List<CloudService>> vendorService(String vendor, Long catalogId, Long tenantId) {
        List<CloudService> cloudServices = cloudServiceRepository.listTenantService(tenantId);
        List<CloudService> vendorService = cloudServices.stream().filter(
                cloudService -> cloudService.getCatalogId().equals(catalogId) && cloudService.getVendorType()
                        .equalsIgnoreCase(vendor)).collect(Collectors.toList());
        return new GeneralResult<>(true, vendorService, "查询失败");
    }

    @Override
    public Result configFlow(Long id, List<ServiceFlowBean> serviceFlowBeans, RequestContext context) {
        // 先解绑服务已经绑定的流程
        flowConfigRepository.deleteById(id);
        ArrayList<ServiceFlowConfig> serviceFlowConfigs = new ArrayList<>();
        for (ServiceFlowBean bean : serviceFlowBeans) {
            if (null != bean.getFlowId() && 0L != bean.getFlowId()) {
                ServiceFlowConfig serviceFlowConfig =
                        new ServiceFlowConfig(id, bean.getFlowId(), new Date(), context.getTarget(),
                                bean.getCategory());
                serviceFlowConfigs.add(serviceFlowConfig);
            }
        }
        if (!serviceFlowConfigs.isEmpty()) {
            serviceFlowConfigs.forEach(flowConfigRepository::save);
        }
        return new Result(true, "流程绑定成功");
    }

    @Override
    public Long getFlowIdByServiceId(Long id, ApplyCategory category) {
        return flowConfigRepository.getServiceFlowId(id, category);
    }

    @Override
    public Result batchConfigFlow(BatchServiceFlowBean serviceFlowBeans, RequestContext context) {
        // 先解绑服务已经绑定的流程s
        List<Long> serviceIds = serviceFlowBeans.getServiceIds();
        flowConfigRepository.deleteByIds(serviceIds);
        ArrayList<ServiceFlowConfig> serviceFlowConfigs = new ArrayList<>();
        List<ServiceFlowBean> serviceFlows = serviceFlowBeans.getServiceFlows();
        for (Long serviceId : serviceIds) {
            for (ServiceFlowBean flowBean : serviceFlows) {
                if (null != flowBean.getFlowId() && 0L != flowBean.getFlowId()) {
                    ServiceFlowConfig serviceFlowConfig =
                            new ServiceFlowConfig(serviceId, flowBean.getFlowId(), new Date(), context.getTarget(),
                                    flowBean.getCategory());
                    serviceFlowConfigs.add(serviceFlowConfig);
                }
            }
        }
        if (!serviceFlowConfigs.isEmpty()) {
            serviceFlowConfigs.forEach(flowConfigRepository::save);
        }
        return new Result(true, "流程批量绑定成功");
    }

    /**
     * 绑定服务
     *
     * @param pluginCode      插件编码
     * @param cloudServiceIds 云服务id
     */
    @Override
    @Transactional
    public void bindingService(String pluginCode, List<Long> cloudServiceIds) {
        Assert.notNull(cloudServiceIds, "service not be null");
        Assert.notNull(pluginCode, "code not be null");
        Boolean flag = cloudServiceRepository.getByCode(pluginCode);
        Assert.isTrue(flag, "该插件不存在");
        List<CloudService> cloudServices = new ArrayList<>(cloudServiceIds.size());
        if (!ListTool.isEmpty(cloudServiceIds)) {
            cloudServices = cloudServiceRepository.listByIds(cloudServiceIds);
        }
        cloudServiceRepository.updatePluginCode(pluginCode);
        cloudServices.forEach(cloudService -> {
            cloudService.setPluginCode(pluginCode);
            cloudServiceRepository.update(cloudService);
        });
    }
}
