package com.bocloud.cos.service;

import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.entity.Contract;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.cos.repository.BillItemRepository;
import com.bocloud.cos.repository.ContractRepository;
import com.bocloud.cos.repository.ServiceBillRepository;
import com.bocloud.cos.utils.IdGenerator;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ContractService {
    @Autowired
    private ContractRepository contractRepository;
    @Autowired
    private ServiceBillRepository serviceBillRepository;
    @Autowired
    private BillItemRepository billItemRepository;

    public GeneralResult<GridBean<Contract>> list(Pager pager, RequestContext context) {
        Integer page = pager.getPage();
        List<Param> params = pager.getParams();
        Integer rows = pager.getRows();
        Map<String, String> sorter = pager.getSorter();

        for (Param pm : pager.getParams()) {
            Map<String, Object> param = pm.getParam();
            if (null != param.get("name")) {
                param.put("contactName", param.get("name"));
                param.remove("name");
            }
            if (context.getCatalog().equals(RequestContext.Catalog.User)) {
                if (null == param.get("tenantId")) {
                    continue;
                }
                param.put("creatorId", param.get("tenantId"));
                param.remove("tenantId");
            }
        }
        sorter.put("gmtCreate", Common.ONE);
        List<Contract> contractList = contractRepository.list(page, rows, sorter, params);
        int count = contractRepository.count(params);
        GridBean<Contract> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), count, contractList);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    public GeneralResult<Contract> detail(Long id) {
        // 校验
        Assert.notNull(id, "id不能为空!");
        Contract contract = contractRepository.query(id);
        if (contract == null) {
            return new GeneralResult<>(false, "合同记录不存在或已被删除!");
        }
        return new GeneralResult<>(true, contract, "查询合同详情成功!");
    }

    @Transactional
    public GeneralResult<Void> create(Contract contract, RequestContext context) {
        // 非空校验
        Assert.notNull(contract.getContactName(), "企业名称不能为空!");
        Assert.notNull(contract.getCompanyAddress(), "企业地址不能为空!");
        Assert.notNull(contract.getContactName(), "联系人不能为空!");
        Assert.notNull(contract.getContactPhone(), "联系电话不能为空!");
        Assert.notNull(contract.getStartTime(), "合同起始日期不能为空!");
        Assert.notNull(contract.getEndTime(), "合同终止日期不能为空!");
        // 订单查询校验(订单ids) 是否支付
        BigDecimal totalMoney = BigDecimal.ZERO;
        if (!CollectionUtils.isEmpty(contract.getBillIds())) {
            for (Long billId : contract.getBillIds()) {
                List<BillItem> billItems = billItemRepository.listByBillId(billId);
                for (BillItem billItem : billItems) {
                    totalMoney = totalMoney.add(billItem.getMoney() == null ? BigDecimal.ZERO : billItem.getMoney());
                }
            }
        }
        contract.setTotalMoney(totalMoney.setScale(2, BigDecimal.ROUND_HALF_UP));
        contract.setTenantId(context.getTenant());
        contract.setGmtCreate(new Date());
        contract.setCreatorId(context.getTarget());
        contract.setStatus(0);
        // 申请编号随便给,后续提供规则后更改
        contract.setApplySn(IdGenerator.getInstance().nextId());
        contractRepository.save(contract);
        if (!CollectionUtils.isEmpty(contract.getBillIds())) {
            for (Long billId : contract.getBillIds()) {
                ServiceBill serviceBill = serviceBillRepository.query(billId);
                serviceBill.setIsContracted(true);
                serviceBill.setContractId(contract.getId());
                serviceBillRepository.update(serviceBill);
            }
        }
        return new GeneralResult<>(true, "新增合同成功!");
    }

    public GeneralResult<Void> delete(Long id) {
        // 校验是否存在
        Assert.notNull(id, "id不能为空!");
        Contract contract = contractRepository.query(id);
        if (contract == null) {
            return new GeneralResult<>(false, "合同记录已被删除或不存在!");
        }
        contractRepository.remove(id);
        // 逻辑删除合同
        return new GeneralResult<>(true, "删除合同记录成功!");
    }

    @Transactional
    public GeneralResult<Void> updateStatus(Long id, String status, Long attachmentId, RequestContext context) {
        Assert.notNull(id, "id不能为空");
        Assert.notNull(status, "类别不能为空");
        Contract existContract = contractRepository.query(id);
        if (existContract == null) {
            return new GeneralResult<>(false, "合同不存在或已被删除!");
        }
        switch (status) {
            case "canceled":
            case "reject":
                // 查询合同的ID
                existContract.setStatus(2);
                billItemRepository.untieContract(id);
                break;
            case "pass":
                existContract.setStatus(1);
                break;
            default:
                break;
        }
        if (null != attachmentId) {
            existContract.setAttachmentId(attachmentId);
        }
        existContract.setGmtModify(new Date());
        existContract.setMenderId(context.getTarget());
        contractRepository.update(existContract);
        return new GeneralResult<>(true, "状态修改成功!");
    }
}
