package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudBill;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.model.ChargeSumSetResponseModel;
import com.bocloud.cos.repository.CloudBillRepository;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.IdGenerator;
import com.megatron.common.encrypt.DESEncryptor;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.bouncycastle.util.encoders.UrlBase64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

@Slf4j
@Service
public class QingCloudBillService {

    @Autowired
    private CloudBillRepository cloudBillRepository;

    private String generateUrl(Map<String, Object> paramMap, String accessKey, String secretAccessKey) {
        // 公共参数
        paramMap.put("time_stamp", URLEncoder.encode(DateUtil.localDateZoneTimeFormat(LocalDateTime.now()), StandardCharsets.UTF_8));
        paramMap.put("access_key_id", accessKey);
        paramMap.put("version", "1");
        paramMap.put("signature_method", "HmacSHA256");
        paramMap.put("signature_version", "1");
        paramMap.put("limit", 100);

        String sign;
        do {
            Map<String, String> stringStringMap = generateSignature(paramMap, secretAccessKey);
            sign = stringStringMap.get("sign");
            if (!sign.contains("_")) {
                return stringStringMap.get("url");
            }
            // 重新设置时间戳
            paramMap.put("time_stamp", URLEncoder.encode(DateUtil.localDateZoneTimeFormat(LocalDateTime.now()), StandardCharsets.UTF_8));
        } while (true);
    }

    private Map<String, String> generateSignature(Map<String, Object> paramMap, String secretAccessKey) {
        try {
            List<String> keyList = new ArrayList<>(paramMap.keySet());
            Collections.sort(keyList);

            StringBuilder stringToSign = new StringBuilder().append("GET\n").append("/iaas/\n");

            StringBuilder urlParam = new StringBuilder();
            for (String key : keyList) {
                urlParam.append(key).append("=").append(paramMap.get(key)).append("&");
            }
            String signStr = urlParam.substring(0, urlParam.length() - 1);
            stringToSign.append(signStr);

            Mac hmacSha256 = Mac.getInstance("HmacSHA256");
            hmacSha256.init(new SecretKeySpec(secretAccessKey.getBytes(StandardCharsets.UTF_8), "HmacSHA256"));
            byte[] signatureBytes = hmacSha256.doFinal(stringToSign.toString().getBytes(StandardCharsets.UTF_8));

            String sign = new String(UrlBase64.encode(signatureBytes)).replace("-", "%2B").replace(".", "%3D");
            String url = "https://api.qingcloud.com/iaas/?" + urlParam.append("signature=").append(sign);

            return Map.of("sign", sign, "url", url);
        } catch (Exception e) {
            log.error("生成签名异常：{}", e);
        }
        return null;
    }

    public void syncBill(List<CloudVendorAccount> vendorAccounts) {
        for (CloudVendorAccount vendorAccount : vendorAccounts) {
            try {
                JSONObject authentication = JSON.parseObject(vendorAccount.getAuthentication());
                String accessKey = new DESEncryptor().decrypt(authentication.getString("accessKey"), "BocloudCMP");
                String secretAccessKey = new DESEncryptor().decrypt(authentication.getString("secretKey"), "BocloudCMP");
                JSONArray array = JSON.parseArray(authentication.getString("regions"));
                String zone = array.get(0).toString();

                // 接口参数
                Map<String, Object> paramMap = new HashMap<>(3);
                paramMap.put("action", "GetChargeSums");
                paramMap.put("zone", zone);
                paramMap.put("duration", "daily");
                paramMap.put("group_by", "resource_id");

                OkHttpClient client = new OkHttpClient().newBuilder().build();
                Request request = new Request.Builder()
                        .url(generateUrl(paramMap, accessKey, secretAccessKey))
                        .method("GET", null)
                        .build();
                Response response = client.newCall(request).execute();
                String result = response.isSuccessful() ? response.body().string() : null;

                List<ChargeSumSetResponseModel> responseModels = new ArrayList<>();
                JSONObject resultObj = JSON.parseObject(result, JSONObject.class);
                JSONObject chargeSumSetObj = JSON.parseObject(resultObj.get("charge_sum_set").toString(), JSONObject.class);
                for (String key : chargeSumSetObj.keySet()) {
                    responseModels.addAll(JSON.parseArray(JSON.toJSONString(chargeSumSetObj.get(key)), ChargeSumSetResponseModel.class));
                }

                List<CloudBill> bills = new ArrayList<>();
                for (ChargeSumSetResponseModel responseModel : responseModels) {
                    Instant instant = Instant.ofEpochMilli(responseModel.getDaily());
                    ZoneId zoneId = ZoneId.systemDefault();
                    LocalDateTime time = LocalDateTime.ofInstant(instant, zoneId);

                    // 如果资源已有当天数据，则不进行插入
                    boolean exist = cloudBillRepository.isExistByResourceId(responseModel.getResource_id(), Date.from(instant));
                    if (exist) {
                        continue;
                    }

                    CloudBill newBill = new CloudBill();
                    newBill.setOrderSn(IdGenerator.getInstance().nextId());
                    newBill.setProductName(responseModel.getResource_type_cn());
                    newBill.setVendorId(vendorAccount.getId());
                    newBill.setVendorType(vendorAccount.getVendorType());
                    newBill.setMonth(DateUtil.localYearMonthFormat(time));
                    newBill.setStatus("paid");
                    newBill.setChargeMode("PostCharge");
                    newBill.setResourceId(responseModel.getResource_id());
                    newBill.setFeeBeginTime(Date.from(instant));
                    newBill.setFeeEndTime(Date.from(instant));
                    newBill.setMoney(new BigDecimal(responseModel.getFee()));
                    newBill.setGmtCreate(Date.from(instant));
                    bills.add(newBill);
                }
                if (bills.isEmpty()) {
                    log.warn("ID为[{}], 名称为[{}]的云平台没有要同步的最新账单数据：{}", vendorAccount.getId(), vendorAccount.getVendorName());
                    return;
                }
                cloudBillRepository.batchSave(bills, bills.size());
            } catch (Exception e) {
                log.error("同步ID为[{}], 名称为[{}]的云平台账单异常：{}", vendorAccount.getId(), vendorAccount.getVendorName(), e);
            }
        }
    }
}
