package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.ServiceAddress;
import com.bocloud.cos.entity.ServiceAddressVendor;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.SmsInternalService;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.cos.intf.ServiceAddressService;
import com.bocloud.cos.params.ServiceAddressParams;
import com.bocloud.cos.repository.ServiceAddressRepository;
import com.bocloud.cos.repository.ServiceAddressVendorRepository;
import com.bocloud.cos.utils.ExportUtil;
import com.bocloud.cos.entity.internel.CloudVendor;
import com.bocloud.cos.entity.internel.Department;
import com.bocloud.cos.entity.internel.Tenant;
import com.bocloud.cos.entity.internel.User;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 云服务业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ServiceAddressServiceImpl implements ServiceAddressService {

    private final LockFactory lockFactory;
    private final ServiceAddressRepository serviceAddressRepository;
    private final ServiceAddressVendorRepository serviceAddressVendorRepository;
    private final ImsInternalService imsInternalService;
    private final SmsInternalService smsInternalService;
    private final RedisTemplate<String, String> redisTemplate;
    private final SmsTenantService smsTenantService;
    private final SmsUserService smsUserService;

    @Override
    public GeneralResult<GridBean<ServiceAddress>> list(Pager pager, RequestContext context) {
        pager.getSorter().put("gmtCreate", Common.ONE);
        GridBean<ServiceAddress> gridBean;
        //权限过滤
        List<Long> departIdList = null;
        if (RequestContext.Catalog.Tenant == context.getCatalog()) {
            //获取租户的组织id
            Tenant tenant = smsTenantService.detail(context.getTenant());
            if (null == tenant) {
                return new GeneralResult<>(false, "查询失败");
            }
            if (null != tenant.getDepartmentId()) {
                departIdList = new ArrayList<>();
                departIdList.add(tenant.getDepartmentId());
                //查出下属的所有组织id
                departIdList.addAll(smsInternalService.getDepartMap(tenant.getDepartmentId()).keySet());
            }
        } else if (RequestContext.Catalog.User == context.getCatalog()) {
            GeneralResult<User> result = smsUserService.detail(context.getTarget());
            if (result.isFailed() || null == result.getData()) {
                return new GeneralResult<>(false, "查询失败");
            }
            //获取组织id
            User user = result.getData();
            if (null != user.getDepartId()) {
                departIdList = new ArrayList<>();
                departIdList.add(user.getDepartId());
                //查出下属的所有组织id
                departIdList.addAll(smsInternalService.getDepartMap(user.getDepartId()).keySet());
            }
        } else {
            departIdList = new ArrayList<>();
        }
        // 没有可查询的组织
        if (null == departIdList) {
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), 0, new ArrayList<>());
            return new GeneralResult<>(true, gridBean, "查询成功");
        }
        // 管理员或者有可查询的组织
        int total = serviceAddressRepository.count(pager.getParams(), departIdList);
        if (pager.getSimple()) {
            List<SimpleBean> beans = serviceAddressRepository.simplelist(pager.getParams(), pager.getSorter(), departIdList);
            gridBean = new GridBean(1, 1, total, beans);
        } else {
            List<ServiceAddress> list = serviceAddressRepository.list(pager, departIdList);
            if (!CollectionUtils.isEmpty(list)) {
                //数据处理
                Map<String, String> dictMap = smsInternalService.dictChildrenList("SERVICE_PROVIDER_TYPE");
                Map<Long, String> departMap = smsInternalService.getDepartMap(null);
                Map<String, String> cityDictMap = smsInternalService.dictChildrenList("CITY");
                // 处理创建者名称
                for (ServiceAddress address : list) {
                    address.setDepartName(departMap.get(address.getDepartId()));
                    address.setTypeName(dictMap.get(address.getType()));
                    address.setCityName(cityDictMap.get(address.getCity()));
                    if (RequestContext.Catalog.Tenant.name().equalsIgnoreCase(address.getCreatorType())) {
                        address.setCreator(
                                redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_" + address.getCreatorId()));
                    } else {
                        address.setCreator(
                                redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + address.getCreatorId()));
                    }
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        }
        return new GeneralResult<>(true, gridBean, "查询成功");
    }


    @Override
    @Transactional
    public Result remove(Long id, Long userId) {
        String path = ServiceAddress.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceAddress service = serviceAddressRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            //删除通讯录
            serviceAddressRepository.remove(ServiceAddress.class, service.getId());
            //删除通讯录的平台
            serviceAddressVendorRepository.removeByAddressId(service.getId());
            return new Result(true, "删除成功");
        } catch (RuntimeException e) {
            throw new RuntimeException("删除失败:" + e.getMessage(), e);
        }
    }

    @Override
    public GeneralResult<ServiceAddress> detail(Long id) {
        ServiceAddress address = serviceAddressRepository.query(id);
        //数据处理
        if (null != address) {
            //查询平台相关信息
            address.setCloudVendorIds(serviceAddressVendorRepository.listVendorIdByAddressId(id));
            //类型
            Map<String, String> dictMap = smsInternalService.dictChildrenList("SERVICE_PROVIDER_TYPE");
            address.setTypeName(dictMap.get(address.getType()));
            //地市
            Map<String, String> cityDictMap = smsInternalService.dictChildrenList("CITY");
            address.setCityName(cityDictMap.get(address.getCity()));
            //部门
            Department depart = smsInternalService.getDepart(address.getDepartId());
            address.setDepartName(null != depart ? depart.getName() : null);
            //创建者名称
            if (RequestContext.Catalog.Tenant.name().equalsIgnoreCase(address.getCreatorType())) {
                address.setCreator(
                        redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_" + address.getCreatorId()));
            } else {
                address.setCreator(
                        redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + address.getCreatorId()));
            }
        }
        return new GeneralResult<>(true, address, "查询成功");
    }

    @Override
    public Result create(ServiceAddressParams params, RequestContext context) {
        boolean flag = serviceAddressRepository.existsName(params.getName(), null);
        if (flag) {
            return new Result(false, "名称已存在");
        }
        flag = serviceAddressRepository.existsCode(params.getCode(), null);
        if (flag) {
            return new Result(false, "编码已存在");
        }
        ServiceAddress address = new ServiceAddress();
        updateServiceAddress(params, address);
        address.setStatus(BocloudStatus.Basic.NORMAL.name());
        address.setGmtCreate(new Date());
        address.setCreatorType(context.getCatalog().name());
        if (RequestContext.Catalog.Tenant == context.getCatalog()) {
            address.setCreatorId(context.getTenant());
        } else {
            address.setCreatorId(context.getTarget());
        }
        serviceAddressRepository.save(address);
        return new Result(true, "新增成功");
    }

    @Transactional
    @Override
    public Result modify(ServiceAddressParams params, Long userId) {
        String path = ServiceAddress.class.getSimpleName() + "_" + params.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceAddress address = serviceAddressRepository.query(params.getId());
            String oldName = address.getName();
            Assert.notNull(address, "数据不存在！");
            if (serviceAddressRepository.existsName(params.getName(), address.getId())) {
                return new Result(false, "名称已存在");
            }
            if (serviceAddressRepository.existsCode(params.getCode(), address.getId())) {
                return new Result(false, "编码已存在");
            }
            updateServiceAddress(params, address);
            address.setMenderId(userId);
            address.setGmtModify(new Date());
            boolean result = serviceAddressRepository.update(address);
            return result ? new Result(true, "修改成功") : new Result(false, "修改失败");
        } catch (RuntimeException e) {
            log.error(" modify service address fail: ", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<List<CloudVendor>> listVendors(Long excludeId) {
        try {
            List<CloudVendor> list = imsInternalService.getVendorList();
            //查询所有已配置的平台
            List<Long> configVendorIds = serviceAddressVendorRepository.listConfigVendorIds(excludeId);
            for (CloudVendor vendor : list) {
                vendor.setIsConfig(configVendorIds.contains(vendor.getId()));
            }
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("list service address vendor failed: ", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Department>> listDepart() {
        try {
            return new GeneralResult<>(true, smsInternalService.listDepart(), "查询成功");
        } catch (Exception e) {
            log.error("list service address depart failed: ", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public Result configVendor(Long id, List<Long> vendorIds, Long userId) {
        String path = ServiceAddress.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceAddress address = serviceAddressRepository.query(id);
            Assert.notNull(address, "数据不存在！");
            List<Long> removeVendorIds = new ArrayList<>();
            List<Long> addVendorIds = new ArrayList<>();
            if (CollectionUtils.isEmpty(vendorIds)) {
                serviceAddressVendorRepository.removeByAddressId(id);
            } else {
                List<Long> configVendorIds = serviceAddressVendorRepository.listVendorIdByAddressId(id);
                removeVendorIds = configVendorIds.stream().filter(num -> !vendorIds.contains(num)).toList();
                addVendorIds = vendorIds.stream().filter(num -> !configVendorIds.contains(num)).toList();
            }
            //删除
            if (!CollectionUtils.isEmpty(removeVendorIds)) {
                serviceAddressVendorRepository.removeByVendorIds(removeVendorIds, id);
            }
            //新增
            if (!CollectionUtils.isEmpty(addVendorIds)) {
                List<ServiceAddressVendor> addList = new ArrayList<>();
                for (Long vendorId : addVendorIds) {
                    ServiceAddressVendor vendor = new ServiceAddressVendor();
                    vendor.setAddressId(id);
                    vendor.setVendorId(vendorId);
                    vendor.setCreatorId(userId);
                    vendor.setGmtCreate(new Date());
                    addList.add(vendor);
                }
                serviceAddressVendorRepository.batchSave(addList, 100);
            }
            return new Result(true, "平台配置成功");
        } catch (RuntimeException e) {
            log.error("config service address vendor failed: ", e);
            return new Result(false, "平台配置失败");
        }
    }


    @Override
    public XSSFWorkbook export(List<Param> params, RequestContext context) {
        try {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Map<String, String> sorter = Maps.newHashMap();
            sorter.put("gmtCreate", Common.ONE);
            List<ServiceAddress> list = new ArrayList<>();
            //权限过滤
            List<Long> departIdList = null;
            if (RequestContext.Catalog.Tenant == context.getCatalog()) {
                //获取租户的组织id
                Tenant tenant = smsTenantService.detail(context.getTenant());
                if (null == tenant) {
                    log.error("导出失败:租户获取失败");
                    return null;
                }
                if (null != tenant.getDepartmentId()) {
                    departIdList = new ArrayList<>();
                    departIdList.add(tenant.getDepartmentId());
                    //查出下属的所有组织id
                    departIdList.addAll(smsInternalService.getDepartMap(tenant.getDepartmentId()).keySet());
                }
            } else if (RequestContext.Catalog.User == context.getCatalog()) {
                GeneralResult<User> result = smsUserService.detail(context.getTarget());
                if (result.isFailed() || null == result.getData()) {
                    log.error("导出失败:用户获取失败");
                    return null;
                }
                //获取组织id
                if (null != result.getData().getDepartId()) {
                    departIdList = new ArrayList<>();
                    departIdList.add(result.getData().getDepartId());
                    //查出下属的所有组织id
                    departIdList.addAll(smsInternalService.getDepartMap(result.getData().getDepartId()).keySet());
                }
            } else {
                departIdList = new ArrayList<>();
            }
            if (null != departIdList) {
                // 管理员或者有可查询的组织
                list = serviceAddressRepository.list(params, sorter, departIdList);
            }
            Map<String, String> dictMap = smsInternalService.dictChildrenList("SERVICE_PROVIDER_TYPE");
            Map<String, String> cityDictMap = smsInternalService.dictChildrenList("CITY");
            Map<Long, String> departMap = smsInternalService.getDepartMap(null);
            // 处理创建者名称
            for (ServiceAddress address : list) {
                address.setDepartName(departMap.get(address.getDepartId()));
                address.setTypeName(dictMap.get(address.getType()));
                address.setCityName(cityDictMap.get(address.getCity()));
                if (RequestContext.Catalog.Tenant.name().equalsIgnoreCase(address.getCreatorType())) {
                    address.setCreator(
                            redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_" + address.getCreatorId()));
                } else {
                    address.setCreator(
                            redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + address.getCreatorId()));
                }
            }

            // 创建HSSFWorkbook
            String sheetName = "服务商数据";
            String[] cellTitle = new String[]{"名称", "联系人", "联系电话", "邮箱", "服务商类型", "所属组织", "编码", "地市", "创建人", "创建时间"};
            String[][] rows;
            if (!ListTool.isEmpty(list)) {
                rows = new String[list.size()][cellTitle.length];
                for (int i = 0; i < list.size(); i++) {
                    int j = 0;
                    ServiceAddress address = list.get(i);
                    rows[i][j++] = address.getName();
                    rows[i][j++] = address.getContactName();
                    rows[i][j++] = address.getContactPhone();
                    rows[i][j++] = address.getContactEmail();
                    rows[i][j++] = address.getTypeName();
                    rows[i][j++] = address.getDepartName();
                    rows[i][j++] = address.getCode();
                    rows[i][j++] = address.getCityName();
                    rows[i][j++] = address.getCreator();
                    rows[i][j] = formatter.format(address.getGmtCreate());
                }
            } else {
                rows = new String[0][cellTitle.length];
            }
            return ExportUtil.getXSSFWorkbook(sheetName, cellTitle, rows);
        } catch (Exception e) {
            log.error("create XSSFWorkBook error,", e);
            return null;
        }
    }

    /**
     * 根据参数修改对象
     *
     * @param params  参数
     * @param address 修改的对象
     */
    private void updateServiceAddress(ServiceAddressParams params, ServiceAddress address) {
        address.setCity(params.getCity());
        address.setType(params.getType());
        address.setCode(params.getCode());
        address.setContactEmail(params.getContactEmail());
        address.setContactPhone(params.getContactPhone());
        address.setContactName(params.getContactName());
        address.setDepartId(params.getDepartId());
        address.setDepartIds(null == params.getDepartIds() ? "[]" : JSONObject.toJSONString(params.getDepartIds()));
        address.setName(params.getName());
    }
}
