package com.bocloud.cos.service;

import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.model.ServiceCatalogTreeVO;
import com.bocloud.cos.repository.ServiceCatalogRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 服务目录业务操作实现
 *
 * @author lym
 * @since 2022年5月30日
 */
@Service
@RequiredArgsConstructor
public class ServiceCatalogService {

    private final ServiceCatalogRepository serviceCatalogRepository;
    private final LockFactory lockFactory;

    /**
     * 添加服务目录
     *
     * @param catalog 服务目录
     * @return 操作结果
     */
    public Result create(ServiceCatalog catalog) {
        Assert.isTrue(StringUtils.isNotBlank(catalog.getName()), "名称不能为空");
        Assert.isTrue(StringUtils.isNotBlank(catalog.getCode()), "编码不能为空");
        boolean existsName = serviceCatalogRepository.existsName(catalog.getName());
        Assert.isTrue(!existsName, "名称已存在");
        boolean existsCode = serviceCatalogRepository.existsCode(catalog.getCode());
        Assert.isTrue(!existsCode, "编码已存在");
        Integer maxId = serviceCatalogRepository.getMaxId();
        if (null == maxId) {
            maxId = 0;
        }
        catalog.setSn(String.format("%02d", maxId));
        serviceCatalogRepository.save(catalog);
        return new Result(true, "新增成功");
    }

    /**
     * 编辑服务目录
     *
     * @param catalog 服务目录
     * @return 操作结果
     */
    public Result modify(ServiceCatalog catalog) {
        String path = ServiceCatalog.class.getSimpleName() + "_" + catalog.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Assert.isTrue(StringUtils.isNotBlank(catalog.getName()), "名称不能为空");
            Assert.isTrue(StringUtils.isNotBlank(catalog.getCode()), "编码不能为空");
            ServiceCatalog serviceCatalog = serviceCatalogRepository.get(ServiceCatalog.class, catalog.getId());
            if (!serviceCatalog.getName().equals(catalog.getName()) && serviceCatalogRepository.existsCode(catalog.getName())) {
                return new Result(false, "名称已存在");
            }
            if (!serviceCatalog.getCode().equals(catalog.getCode()) && serviceCatalogRepository.existsCode(catalog.getCode())) {
                return new Result(false, "编码已存在");
            }
            serviceCatalog.setName(catalog.getName());
            serviceCatalog.setCode(catalog.getCode());
            serviceCatalog.setSort(catalog.getSort());
            serviceCatalog.setRemark(catalog.getRemark());
            serviceCatalog.setMenderId(catalog.getMenderId());
            serviceCatalogRepository.update(serviceCatalog);
            return new Result(true, "编辑成功");
        }
    }

    /**
     * 删除服务目录
     *
     * @param id      ID
     * @param context 当前用户
     * @return 操作结果
     */
    public Result remove(Long id, RequestContext context) {
        String path = ServiceCatalog.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 校验该目录下是否有子目录或者服务，如果则不允许删除
            List<ServiceCatalog> serviceCatalogs = serviceCatalogRepository.queryChildren(id);
            if (!serviceCatalogs.isEmpty()) {
                return new Result(false, "当前服务目录有子节点，不可删除");
            }
            ServiceCatalog serviceCatalog = serviceCatalogRepository.get(ServiceCatalog.class, id);
            serviceCatalog.setMenderId(context.getTarget());
            serviceCatalogRepository.remove(ServiceCatalog.class, id);
            return new Result(true, "删除成功");
        }
    }

    /**
     * 获取服务目录详情
     *
     * @param id ID
     * @return 服务目录
     */
    public ServiceCatalog detail(Long id) {
        return serviceCatalogRepository.get(ServiceCatalog.class, id);
    }

    /**
     * 获取服务目录树
     *
     * @return 服务目录树
     */
    public GeneralResult<List<ServiceCatalogTreeVO>> tree() {
        List<ServiceCatalog> serviceCatalogs = serviceCatalogRepository.listAll();
        if (serviceCatalogs.isEmpty()) {
            return new GeneralResult<>(true, new ArrayList<>(), "查询成功");
        }
        List<ServiceCatalogTreeVO> list = generateTree(serviceCatalogs, serviceCatalogs, new HashSet<>());
        list = list.stream().sorted(Comparator.comparing(ServiceCatalogTreeVO::getSort)).collect(Collectors.toList());
        return new GeneralResult<>(true, list, "查询成功");
    }

    private List<ServiceCatalogTreeVO> generateTree(List<ServiceCatalog> oldCatalogs, List<ServiceCatalog> newCatalogs, Set<Long> ids) {
        List<ServiceCatalogTreeVO> result = new ArrayList<>();
        for (ServiceCatalog sc : newCatalogs) {
            if (!ids.add(sc.getId())) {
                continue;
            }
            ServiceCatalogTreeVO scv = new ServiceCatalogTreeVO();
            scv.setId(sc.getId());
            scv.setCatalogName(sc.getName());
            scv.setSort(sc.getSort());
            scv.setName(sc.getName());
            scv.setSn(sc.getSn());
            scv.setCode(sc.getCode());
            scv.setHasCategory(sc.getHasCategory());
            scv.setGmtCreate(sc.getGmtCreate());
            scv.setGmtModify(sc.getGmtModify());
            scv.setProps(sc.getProps());
            scv.setRemark(sc.getRemark());
            scv.setParentId(sc.getParentId());
            scv.setCreatorId(sc.getCreatorId());
            scv.setMenderId(sc.getMenderId());
            List<ServiceCatalog> collect = oldCatalogs.stream().filter(t -> sc.getId().equals(t.getParentId())).collect(Collectors.toList());
            collect = collect.stream().sorted(Comparator.comparing(ServiceCatalog::getSort)).collect(Collectors.toList());
            if (!collect.isEmpty()) {
                scv.setChildren(generateTree(oldCatalogs, collect, ids));
            }
            result.add(scv);
        }
        return result;
    }
}
